// Copyright (C) 2003 Intel Corporation

//////////////////////////////////////////////////////////////////////////
// EventWin32.cpp
//
// This file contains the win32 impelmentation of the Event class
//////////////////////////////////////////////////////////////////////////
#include "Event.h"
#include <windows.h>

class OSEvent {
public:
	HANDLE handle;
};

Event::Event(bool manual)
{
	_osEvent = new OSEvent;
	_osEvent->handle = CreateEvent(0, (manual==true), FALSE, 0);
}

Event::Event(const Event& rhs)
{
	_osEvent = new OSEvent;
	DuplicateHandle(GetCurrentProcess(), rhs._osEvent->handle, 
		GetCurrentProcess(), &_osEvent->handle, 0, 0, DUPLICATE_SAME_ACCESS);
}

Event::~Event()
{
	CloseHandle(_osEvent->handle);
	delete _osEvent;
}

bool Event::wait(unsigned long msecs)
{
	return (WaitForSingleObject (_osEvent->handle, msecs) == WAIT_OBJECT_0);
}

void Event::set()
{
	SetEvent (_osEvent->handle);
}

void Event::reset()
{
	ResetEvent (_osEvent->handle);
}


void Event::pulse()
{
	PulseEvent (_osEvent->handle);
}


Event& Event::operator= (const Event& rhs)
{
	if (&rhs != this)
	{
		CloseHandle(_osEvent->handle);
		DuplicateHandle(GetCurrentProcess(), rhs._osEvent->handle, GetCurrentProcess(), 
			&_osEvent->handle, 0, 0, DUPLICATE_SAME_ACCESS);
	}
	return *this;
}
