// Copyright (C) 2003 Intel Corporation

#include "RWLock.h"

RWLock::RWLock() : _counter(0) {}

void RWLock::acquire(const RWMode mode_p) 
{
 
    if (mode_p == READ_ONLY) {
        //wait for writer's exit
        _writeSem.acquire(); 
        //might be blocked only to decrement _counter and _readSem.release() in release() 
        _countSem.acquire(); 
        int tmp = ++_counter;
        _countSem.release(); 
        if (tmp == 1) { 
            //never blocks, no writers, first reader
            _readSem.acquire();  
		}
        _writeSem.release(); 
	} else {
        _writeSem.acquire();
        //wait for reader's exit
        _readSem.acquire(); 
    }
    
    return;
}

//will do nothing if no read-write lock was acquired
void RWLock::switch2RO() 
{
    _countSem.acquire();
    if (!_counter) {
       _counter = 1;
       _writeSem.release(); 
	} 
    _countSem.release(); 
}

//will do nothing if no lock was acquired
void RWLock::release() 
{

    //might be blocked only to increment _counter in read-only mode 
    _countSem.acquire();
	if (_counter) {
        //read-only mode
        _counter--;  
        if (!_counter) {
            //the last reader
            _readSem.release();
		} 
	} else {
        //read-write mode
        _readSem.release();
        _writeSem.release(); 
    }
    _countSem.release();  

    return;
}





