// Copyright (C) 2003 Intel Corporation

//////////////////////////////////////////////////////////////////////////
// SemaphoreWin32.cpp
//
// This file contains the win32 ipmlementation of the Semaphore class
//////////////////////////////////////////////////////////////////////////
#include "Semaphore.h"
#include <windows.h>

class OSSemaphore {
public:
	HANDLE handle;
};

Semaphore::Semaphore(int maxval) 
{
	_osSemaphore = new OSSemaphore;
	_osSemaphore->handle = CreateSemaphore(0, maxval, maxval, 0);
}

Semaphore::Semaphore(const Semaphore& rhs)
{
	_osSemaphore = new OSSemaphore;
	DuplicateHandle(GetCurrentProcess(), rhs._osSemaphore->handle, GetCurrentProcess(), 
		&_osSemaphore->handle, 0, 0, DUPLICATE_SAME_ACCESS);
}

Semaphore::~Semaphore()
{
	CloseHandle(_osSemaphore->handle);
	delete _osSemaphore;
}

void Semaphore::acquire()
{
	// blocks until value !=0, and decrements the value
	WaitForSingleObject (_osSemaphore->handle, INFINITE);
}

void Semaphore::release()
{
	// increments the value by 1
	ReleaseSemaphore (_osSemaphore->handle, 1, NULL);
}

bool Semaphore::acquireTry()
{
	// try to acquire the semaphore: return 'true' if succeded
	return (WaitForSingleObject (_osSemaphore->handle, 0) == WAIT_OBJECT_0);
}

Semaphore& Semaphore::operator= (const Semaphore& rhs)
{
	if (&rhs != this)
	{
		CloseHandle(_osSemaphore->handle);
		DuplicateHandle(GetCurrentProcess(), rhs._osSemaphore->handle, GetCurrentProcess(), 
			&_osSemaphore->handle, 0, 0, DUPLICATE_SAME_ACCESS);
	}
	return *this;
}
