// Copyright (C) 2003 Intel Corporation

//////////////////////////////////////////////////////////////////////////
// ThreadWin32.cpp
//
// This file contains the win32 implementation of the Thread class
///////////////////////////////////////////////////////////////////////////
#include "Thread.h"
#include <windows.h>
#include <process.h>

class OSThread {
public:
	HANDLE handle;
	static unsigned int __stdcall threadFunc(void* p);
};

unsigned int __stdcall OSThread::threadFunc(void *p)
{
	((Thread*)p)->run();
	return 0;
}

Thread::Thread (CallbackFunction func, void* param)
{
	_func = func;
	_param = param;
	_osThread = new OSThread;
	_osThread->handle = (HANDLE)_beginthreadex(0, 0, OSThread::threadFunc, (void*)this, CREATE_SUSPENDED, 0);
}

Thread::Thread(const Thread& rhs)
{
	_func = rhs._func;
	_param = rhs._param;
	_startTime = rhs._startTime;
	_osThread = new OSThread;
	DuplicateHandle(GetCurrentProcess(), rhs._osThread->handle, GetCurrentProcess(), 
		&_osThread->handle, 0, 0, DUPLICATE_SAME_ACCESS);
}

Thread::~Thread()
{
	CloseHandle(_osThread->handle);
	delete _osThread;
}

unsigned long Thread::currentThread()
{
	return GetCurrentThreadId();
}

bool Thread::wait(unsigned long msecs) const
{
	return (WaitForSingleObject (_osThread->handle, msecs) == WAIT_OBJECT_0);
}

bool Thread::start()
{
	if (_osThread->handle == 0) {
		return false;
	}
	_startTime = GetTickCount();
	ResumeThread(_osThread->handle);
	return true;
}

bool Thread::running() const
{
	return (WaitForSingleObject (_osThread->handle, 0) != WAIT_OBJECT_0);
}

void Thread::msleep(long msecs)
{
	Sleep(msecs);
}

void Thread::run()
{
	if (_func != NULL) {
		_func(_param);
	}
}

long Thread::elapsedTime() const
{
	return GetTickCount() - _startTime;
}

Thread& Thread::operator= (const Thread& rhs)
{
	if (&rhs != this)
	{
		CloseHandle(_osThread->handle);
		DuplicateHandle(GetCurrentProcess(), rhs._osThread->handle, GetCurrentProcess(), 
			&_osThread->handle, 0, 0, DUPLICATE_SAME_ACCESS);
		_startTime = rhs._startTime;
		_func = rhs._func;
		_param = rhs._param;
	}
	return *this;
}

