// Copyright (C) 2003 Intel Corporation

//////////////////////////////////////////////////////////////////////////
// Timer.cpp
//
// This file contains the implementation of the Timer class
//////////////////////////////////////////////////////////////////////////
#include "Timer.h"

#define TIMER_NOT_REGISTERED -1

Timer::Timer (CallbackFunction func, void* param) : _func(func), _param(param), _timerId(TIMER_NOT_REGISTERED) 
{
}

Timer::~Timer() 
{
	stop();
}

void Timer::start(long msecs, bool recurring)
{
	if (_timerId == TIMER_NOT_REGISTERED) {
		// timer is not already registered
		SPtr<CallbackParam> p(new _CallbackParam(this));
		_timerId = TimerManager::instance()->registerCallback(msecs, _internalCallback, p, recurring);
	}
}

void Timer::stop()
{
	if (_timerId != TIMER_NOT_REGISTERED) {
		// timer is registered
		TimerManager::instance()->unregisterCallback(_timerId);
		_timerId = TIMER_NOT_REGISTERED;
	}
}

void Timer::run()
{
	if (_func != NULL) {
		_func(_param);
	}
}

void Timer::_internalCallback(SPtr<CallbackParam> p)
{
	if( !p.get() ){
		return;
	}
	Timer* t = (static_cast<_CallbackParam*>(p.get()))->self;
	t->run();
}

#ifndef __IMR_WIN__
struct timeb Timer::_base = {0,};
Semaphore Timer::_sem;
unsigned long Timer::tickCount()
{
	Lock l(_sem);

	//static unsigned long base = 0;
	struct timeb tb;
	unsigned long res;

	if (_base.time == 0) {
		ftime(&_base);
	}

	ftime(&tb);
	res = (tb.time - _base.time) * 1000;
	res += tb.millitm;

	return res;
}
#endif

