//  Copyright (C) 2007 Intel Corporation
#ifndef _AUTHENTICATION_DEFINITIONS_H
#define _AUTHENTICATION_DEFINITIONS_H

#include <string>
using namespace std;

// Authentication status codes.
// Contains also not-authentication proper codes, to fit other errors which 
//  might be received during the authentication process.
typedef enum
{
	AuthenticationResultSuccess = 0,
	AuthenticationResultFailure = 1,
	AuthenticationResultAuditLogFailure = 2,
	AuthenticationResultClientBusy = 3,
	AuthenticationResultUnsupported = 4,
}AuthenticationResult;

#define MAX_SESSION_PACKET_LENGTH 12000
// disable the "zero-sized array" warning
#pragma warning(disable:4200)

// timeout for responses to explicit message requests (e.g. StartRedirection)
//#define TCP_SOL_TIMEOUT 20000

/* Link preference flags */
#define LINK_PREFERENCE_NO_SUPPORT 0
#define LINK_PREFERENCE_SUPPORT_EXISTS 1

#define RECEIVE_TIMEOUT 20000

// should be changed to KVMR
static const string START_REDIRECTION_PROTOCOL = "KVMR";

/* Message Types */
#define StartRedirectionSession      0x10
#define StartRedirectionSessionReply 0x11
#define EndRedirectionSession        0x12
#define AuthenticateSession          0x13
#define AuthenticateSessionReply     0x14

#define StartKVMRedirection			 0x40
#define StartKVMRedirectionReply	 0x41

/*StartRedirectionSessionReply Statuses */
#define STATUS_SUCCESS		0x00
#define STATUS_TYPE_UNKNOWN 0x01
#define STATUS_BUSY			0x02
#define STATUS_UNSUPPORTED	0x03
#define STATUS_ERROR		0xff

/* Authentication Types */
#define AUTH_TYPE_QUERY    0x00
#define AUTH_TYPE_BASIC    0x01
#define AUTH_TYPE_KERBEROS 0x02
#define AUTH_TYPE_DIGEST   0x03
#define AUTH_TYPE_DIGEST2  0x04

#define AUTH_STATUS_SUCCESS					 0x00
#define AUTH_STATUS_FAILURE					 0x01
#define AUTH_STATUS_UNSUPPORTED				 0x02
#define AUTH_STATUS_AUDIT_LOG_FULL_OR_LOCKED 0x03

#define MAX_USERNAME_LEN 32
#define MAX_PASSWORD_LEN 32
#define MAX_USERPASS_LEN (MAX_USERNAME_LEN + MAX_PASSWORD_LEN + 2)

#define MAX_LENGTH_USER_NAME_DIGEST	32
#define MAX_LENGTH_REALM_DIGEST		64
#define MAX_LENGTH_NONCE_DIGEST		32
#define MAX_LENGTH_SERVICE_DIGEST	32
#define MAX_LENGTH_CNONCE_DIGEST	32
#define MAX_LENGTH_NC_DIGEST		32
#define MAX_LENGTH_RESPONSE_DIGEST	32
#define MAX_LENGTH_QOP_DIGEST	8

static const string RedirectionService = "/RedirectionService";

#pragma pack(1)

typedef struct {
	UINT8 type;
	union
	{
		UINT8 status;
		UINT8 linkPrefernce;
	};
	UINT8 reserved[2];
} MsgHdr;

typedef struct {
	MsgHdr hdr;
	UINT8 SessionType[4];
} MSG_StartRedirectionSession;

typedef struct {
	MsgHdr hdr;
	union {
		struct {
			UINT8 ProtRevMajor;
			UINT8 ProtRevMinor;
			UINT8 FWRevMajor;
			UINT8 FWRevMinor;
			UINT32 IANA;
			UINT8 OEMDataLength;
			UINT8 OEMData[0];
		} success;

		struct {
			UINT8 activeAddr[4];
			UINT8 reserved[5];
		} busy;

		UINT8 reserved[9];
	} u;
} MSG_StartRedirectionSessionReply;

typedef struct {
	MsgHdr hdr;
} MSG_EndRedirectionSession;

typedef struct {
	MsgHdr hdr;
	UINT8 authType;
	UINT32 dataLength;
	UINT8 data[0];
} MSG_AuthenticateSession, MSG_AuthenticateSessionReply;

typedef struct {
	UINT8	KVMMessageType;
	UINT8	Reserved1;
	UINT8	Reserved2;
	UINT8	Reserved3;
	union
	{
		UINT32	Reserved4;
		UINT32	Status;
	}u;
}MSG_StartKVMRedirection, MSG_StartKVMRedirectionReply;

typedef struct {
	UINT8	UserNameLength;
	UINT8	UserName[MAX_LENGTH_USER_NAME_DIGEST];
	UINT8	RealmLength;
	UINT8	Realm[MAX_LENGTH_REALM_DIGEST];
	UINT8	NonceLength;
	UINT8	Nonce[MAX_LENGTH_NONCE_DIGEST];
	UINT8	ServiceLength;
	UINT8	Service[MAX_LENGTH_SERVICE_DIGEST];
	UINT8	CNonceLength;
	UINT8	CNonce[MAX_LENGTH_CNONCE_DIGEST + 1];
	UINT8	NCLength;
	UINT8	NC[MAX_LENGTH_NC_DIGEST];
	UINT8	ResponseLength;
	UINT8	Response[MAX_LENGTH_RESPONSE_DIGEST + 1];
}MSG_DigestAuthentication;

typedef struct {
	MSG_DigestAuthentication DigestAuth;
	UINT8 QopLength;
	UINT8 Qop[MAX_LENGTH_QOP_DIGEST];
}MSG_DigestAuthentication2;


#pragma pack()

#endif
