//  Copyright (C) 2007 Intel Corporation
#ifndef _AUTHENTICATOR_DIGEST_H
#define _AUTHENTICATOR_DIGEST_H

#include "Authenticator.h"



class AuthenticatorDigest :
	public Authenticator
{
public:
	
	typedef unsigned char HASH_MD_5[16];
	AuthenticatorDigest(SPtr<Socket> socket, string user, string pwd, UINT8 digestType = AUTH_TYPE_DIGEST);
	~AuthenticatorDigest(void);
	AuthenticationResult Authenticate();
	AuthenticationResult Authenticate2();
	

private:
	typedef enum _DIGEST_FIELDS
	{
		DIGEST_FIELD_USERNAME = 0,
		DIGEST_FIELD_REALM,
		DIGEST_FIELD_NONCE,
		DIGEST_FIELD_SERVICE,
		DIGEST_FIELD_CNONCE,
		DIGEST_FIELD_NC,
		DIGEST_FIELD_RESPONSE,
		DIGEST_FIELD_QOP
	}DIGEST_FIELDS;

	AuthenticationResult	parseBufferLength(UINT8* buffer, UINT32& buffer_length,
								  MSG_DigestAuthentication& digestResponse, 
								  DIGEST_FIELDS field, UINT32& OUT_indexBuffer);
	AuthenticationResult	parseBufferLength(UINT8* buffer, UINT32& buffer_length,
								  MSG_DigestAuthentication2& digestResponse, 
								  DIGEST_FIELDS field, UINT32& OUT_indexBuffer);
	AuthenticationResult authFirst(MSG_DigestAuthentication& digestResponse);
	AuthenticationResult authFirst(MSG_DigestAuthentication2& digestResponse);
	AuthenticationResult parseDigestResponse(PacketBuffer& packet, 
								   MSG_DigestAuthentication& digestResponse);
	void fillBuffer(PacketBuffer& packet, MSG_DigestAuthentication& request);
	void fillBuffer(PacketBuffer& packet, MSG_DigestAuthentication2& request);
	
	AuthenticationResult authSecond(MSG_DigestAuthentication& digestResponse, char* qop = "");
	AuthenticationResult authSecond(MSG_DigestAuthentication2& digestResponse);

	AuthenticationResult parseDigestResponseRealmNonce(PacketBuffer& packet, 
													MSG_DigestAuthentication& digestResponse);

	AuthenticationResult parseDigestResponseRealmNonce(PacketBuffer& packet, 
													MSG_DigestAuthentication2& digestResponse);
	static void GenerateCNonce(char *outbuff);
	static void printDigestResponse(MSG_DigestAuthentication& digestResponse);

	string m_user;
	string m_pwd;

	UINT8 m_digestType;
};

#endif /*_AUTHENTICATOR_DIGEST_H*/
