//  Copyright (C) 2007 Intel Corporation
#ifndef _LOGGER_H
#define _LOGGER_H

#include "Timer.h"
#include "SPtr.h"
#include "Lock.h"
#include <fstream>


using namespace std;

class Logger
{
public:
	enum LogSeverity {LOG_INFO = 0, LOG_WARNING = 1, LOG_ERROR = 2};

private:
	Logger(void);
	Logger(Logger&);
	Logger& operator=(Logger&);
	Semaphore			_sem;
	ofstream			_file;
	static	bool		s_EnableLogger;
	static	Semaphore	s_LockEnableLogger;
	void GetTime(char* obj);
	Logger& PrintDetails(LogSeverity severity);
public:
	~Logger(void);
	static Logger& GetInstance(LogSeverity severity = LOG_INFO);
	
	static void EnableLogger(bool enable = true);

	template<class T>
	Logger& operator<< (T& msg)
	{
		{
			Lock l(Logger::s_LockEnableLogger);
			if(!Logger::s_EnableLogger)
				return *this;
		}

		Lock l(_sem);
		_file << msg;
		return *this;
	}

	template<class T>
	Logger& operator<< (const T& msg)
	{
		{
			Lock l(Logger::s_LockEnableLogger);
			if(!Logger::s_EnableLogger)
				return *this;
		}

		Lock l(_sem);
		_file << msg;
		return *this;
	}

	Logger& operator<< (ostream& (*f)(ostream&))
	{
		{
			Lock l(Logger::s_LockEnableLogger);
			if(!Logger::s_EnableLogger)
				return *this;
		}

		Lock l(_sem);
		_file << f;
		return *this;
	}
};

#endif /*_LOGGER_H*/
