//  Copyright (C) 2007 Intel Corporation
#ifndef _SAFE_QUEUE_H
#define _SAFE_QUEUE_H

#include <queue>
#include "Event.h"
#include "Lock.h"
#include "Semaphore.h"

using namespace std;

template <class T>
class SafeQueue 
{
public:
	SafeQueue(void){}
	~SafeQueue(void){}
	void push(T t);
	T pop();
	void clear();
	bool empty();
private:
	Event		_ev;
	Semaphore	_sem;
	queue<T>	_queue;
};

template <class T>
void SafeQueue<T>::push(T t)
{
	Lock l(_sem);
	_queue.push(t);
	_ev.set();
}

template <class T>
T SafeQueue<T>::pop()
{
	T t;
	while(true)
	{
		{
			Lock l(_sem);
			bool a = _queue.empty();
			if(!a)
			{
				_ev.reset();
				t = _queue.front();
				_queue.pop();
				return t;
			}
		}
		_ev.wait();
	}
}

template <class T>
void SafeQueue<T>::clear()
{
	Lock l(_sem);
	while(!_queue.empty())
		_queue.pop();
}

template <class T>
bool SafeQueue<T>::empty()
{
	Lock l(_sem);
	return _queue.empty();
}

#endif
