/****************************************************************
Copyright  2010, Intel Corporation. All rights reserved.
*****************************************************************/
#ifndef __SANITIZED_STR_H
#define __SANITIZED_STR_H
#include <string>
#ifdef _WIN32
#ifndef WINDOWS_H
#define WINDOWS_H
#include <windows.h>
#endif
#else
#include "stdLinux.h"
#endif
using std::basic_string;

template <typename CTYPE>
class SanitizedStr
{
public:
	SanitizedStr() : _s() {}
	SanitizedStr(const basic_string<CTYPE>& s) : _s(s) {}
	virtual ~SanitizedStr() {sanitize(_s);}
	basic_string<CTYPE> str() const {return _s;}

	void sanitize() {sanitize(_s);} 
	static void sanitize(basic_string<CTYPE>& s) {
		if (s.empty())
			s.push_back(0x32);
		SecureZeroMemory(&s[0], s.capacity()*sizeof(CTYPE)); 
		s.clear();}
private:
	basic_string<CTYPE> _s;

};
#endif /* __SANITIZED_STR_H */
