/****************************************************************
Copyright  2010, Intel Corporation. All rights reserved.
*****************************************************************/
#ifndef __SEC_STRING_H
#define __SEC_STRING_H

#include <string>
#include <vector>
#include "SanitizedStr.h"
#include "TypeDefinitions.h"

using std::basic_string;
using std::vector;

template <typename CTYPE>
class SecString
{
private:
	bool _valid;
	vector<unsigned char> _s;
public:
	// Ctosr / Dtors
	SecString() : _valid(true), _s() {}
	SecString(basic_string<CTYPE>& s);
	virtual ~SecString() {if (isValid()) sanitize(_s);}

	// Encryption functions
	bool decrypt(SanitizedStr<CTYPE>&  outStr) const;

	// String Functions
	bool isValid() const { return _valid; }
	bool empty() const {return _s.empty(); }
	void clear() { sanitize(_s); _s.clear(); _valid = true;}

	// Friends
	/*inline friend bool operator==<>(const SecString& lhs, const SecString& rhs) {
		return lhs.isValid() && rhs.isValid() && lhs._s == rhs._s;}
	inline friend bool operator!=(const SecString& lhs, const SecString& rhs) {
		return lhs.isValid() && rhs.isValid() && lhs._s != rhs._s;}*/

	inline friend bool operator==(const SecString<CTYPE>& lhs, const SecString<CTYPE>& rhs) {
	return lhs.isValid() && rhs.isValid() && lhs._s == rhs._s;}
	inline friend bool operator!=(const SecString<CTYPE>& lhs, const SecString<CTYPE>& rhs) {
	return lhs.isValid() && rhs.isValid() && lhs._s != rhs._s;}



	string ToBase64String() const;
	void FromBase64String(const string& base);

	// static utils
	// sanitize a string. To be used on the original string after ceating a
	// SecString form it
	static void sanitize(basic_string<CTYPE>& s);
	//static void sanitize(vector<CTYPE>& v);
	static void sanitize(vector<unsigned char>& v);	
};

#endif /* __SEC_STRING_H */
