/****************************************************************
Copyright  2003-2010, Intel Corporation. All rights reserved.
*****************************************************************/
#ifndef __STRING_UTILS_H
#define __STRING_UTILS_H
#include <string>
#include <sstream>
#include <vector>
#include "stdOS.h"
using namespace std;


namespace StrUtl
{
	string	ClearChars(const string& inString, const string& charsToClear);
	wstring	wClearChars(const wstring& inString, const wstring& charsToClear);
	string	ClearWhiteSpace(const string& inString);
	wstring	ClearWWhiteSpace(const wstring& inString);
	string	TrimString(const string& inString);
	wstring	TrimWString(const wstring& inString);
	bool getRandomByte(BYTE &RandomBuf);
	template <typename CTYPE> bool Encrypt(const basic_string<CTYPE>& input,
										   std::vector<unsigned char> &output,
										   const bool& binary = false);
	template <typename CTYPE> bool Decrypt(const std::vector<unsigned char>& input,
										   basic_string<CTYPE>& output);
	bool Encrypt(const std::string& input,std::string &output, const bool& binary = false);
	bool Decrypt(const std::string& input,std::string &output);
	wstring getRandomPassword(const size_t len);
	bool isStrongPassword(const wstring & pwd, const bool underscoreValid);
	bool CompareIgnoreCase(const string& string1, const string& string2); 
	bool CompareIgnoreCase(const wstring& string1, const wstring& string2);
	bool IsDecimalUInt(const string& str);
	bool ContainsChar(const string& str, const string& chars);
	bool IsPrintableAscii(const string& str);
	bool StartsWith(const string& str1, const string& str2);
	bool ConvertStringSidToUserName(std::wstring stringSID,std::wstring& UserNameAndDomain,std::wstring& SystemErrorString);
	DWORD FormatSystemMessage(DWORD errorCode, std::wstring& errorString);
	wstring cleanNonPrintable(wstring text);
	bool pid_check(const wstring& s);
	string ToUpper(const string& str);
	wstring ToWUpper(const wstring& str);
	string ToLower(const string& str);
	wstring ToWLower(const wstring& str);
	wstring GetDomainFromFqdn(const wstring& fqdn);
	wstring GetHostNameFromFqdn(const wstring& fqdn);
	wstring ExtractDomainFromADOU(const wstring& adou, bool fqdnFormat = true);
	int CompareVersions(const string& version1, const string& version2);
	std::vector<unsigned char> ConvertStrToByteArray(const std::string & str);
	string ConvertByteArrayToStr(const std::vector<unsigned char>& arr);
	string DiscardNonHex(const string & str);

	template <class T>
	inline string ToStr(const T& t)
	{
		stringstream tmp;
		tmp << t;
		return tmp.str();
	}
	template<>
	inline string ToStr(const wstring& t)
	{
		size_t len = t.length();
		if (len < 1)
			return string();
		string buff(len, '\0');
		size_t count;
		::wcstombs_s(&count, &buff.at(0), len+1, t.c_str(), len+1);
		if (count < len)
			buff.resize(count);
		return buff;
	}
	template <class T>
	inline string ToStrHex(const T& t)
	{
		stringstream tmp;
		tmp << hex << showbase << t;
		return tmp.str();
	}
	template <class T>
	static wstring ToWStr(const T& t)
	{
		wstringstream tmp;
		tmp << t;
		return tmp.str();
	}
	static wstring ToWStr(const string& t)
	{
		size_t len = t.length();
		if (len < 1)
			return wstring();
		wstring buff(len, L'\0');
		size_t count;
		::mbstowcs_s(&count, &buff.at(0), len+1, t.c_str(), len+1);
		if (count < len)
			buff.resize(count);
		return buff;
	}

	template <class T>
	static wstring ToWStrHex(const T& t)
	{
		wstringstream tmp;
		tmp << hex << showbase << t;
		return tmp.str();
	}
	template <class T>
	static bool FromString(const string& str, T& t)
	{
		stringstream tmp(str);
		tmp >> t;
		return !tmp.fail();
	}
	template <class T>
	static bool FromwString(const wstring& str, T& t)
	{
		wstringstream tmp(str);
		tmp >> t;
		return !tmp.fail();
	}
	static string ToStr(const wchar_t* InString)
	{
		wstring x = InString;
		return ToStr( x );
	}
	static wstring ToWStr(const char * InString)
	{
		string x = InString;
		return ToWStr( x );
	}

	static string ToStr(wchar_t* InString)
	{
		wstring x = InString;
		return ToStr( x );
	}
	static wstring ToWStr(char * InString)
	{
		string x = InString;
		return ToWStr( x );
	}

};
#endif /* __STRING_UTILS_H */
