//  Copyright (C) 2007 Intel Corporation
#include "AuthenticatorBasic.h"
#include "AuthenticationDefinitions.h"

AuthenticatorBasic::AuthenticatorBasic(SPtr<Socket> socket, string user, string pwd) : Authenticator(socket)
{
	m_user = user;
	m_pwd = pwd;
}

AuthenticatorBasic::~AuthenticatorBasic(void)
{
}

AuthenticationResult AuthenticatorBasic::Authenticate()
{
	PacketBuffer packet;
	packet.Memset();
	prepareAuthMsg(&packet, AUTH_TYPE_BASIC);

	MSG_AuthenticateSession* authMsg;
	MSG_AuthenticateSessionReply* authMsgReply;	

	authMsg = (MSG_AuthenticateSession*)packet.data;

	authMsg->dataLength = static_cast<UINT32>((m_user.size() + m_pwd.size())) + 2;
	if (authMsg->dataLength - (packet.size() - sizeof(authMsg->dataLength) - sizeof(authMsg->hdr)) > 0)
	{
		return AuthenticationResultFailure;
	}
	memset(authMsg->data, 0, authMsg->dataLength);
	packet.length += authMsg->dataLength;

	UINT32 user_len = static_cast<UINT32>(m_user.size());
	authMsg->data[0] = user_len;
	if (memcpy_s(&(authMsg->data[1]), authMsg->dataLength - 1, m_user.c_str(), m_user.length()))
	{
		return AuthenticationResultFailure;
	}

	UINT32 pass_len = static_cast<UINT32>(m_pwd.size());
	authMsg->data[(user_len) + 1] = pass_len;
	if (memcpy_s(&(authMsg->data[(user_len)+2]), authMsg->dataLength - ((user_len)+2), m_pwd.c_str(), m_pwd.length()))
	{
		return AuthenticationResultFailure;
	}

	m_socket->send(packet.data, packet.length);

	packet.Memset();

	packet.length = m_socket->receive(packet.data, sizeof(MSG_AuthenticateSessionReply), RECEIVE_TIMEOUT);
	if(packet.length != sizeof(MSG_AuthenticateSessionReply))
		return AuthenticationResultFailure;

	authMsgReply = (MSG_AuthenticateSessionReply*)packet.data;
	
	return AMTRes2AuthenticationResult(authMsgReply->hdr.status);
}