//  Copyright (C) 2007 Intel Corporation
#include "Logger.h"
#include "Timer.h"
#include <time.h>

const char* LOG_FILE_NAME = "log.txt";
const int TIME_LENGTH = 64;

bool	  Logger::s_EnableLogger(false);
Semaphore Logger::s_LockEnableLogger;

Logger::Logger()
{
	Lock l(Logger::s_LockEnableLogger);
	if(Logger::s_EnableLogger)
	{
		_file.open(LOG_FILE_NAME, ios::out);
	}
}

Logger& Logger::GetInstance(Logger::LogSeverity severity)
{
	static Logger logger;
	return logger.PrintDetails(severity);
}

Logger& Logger::PrintDetails(Logger::LogSeverity severity)
{
	{
		Lock l(Logger::s_LockEnableLogger);
		if(!Logger::s_EnableLogger)
			return *this;
	}

	char t[64];
	memset(t, 0, 64);
	GetTime(t);

	Lock l(_sem);
	switch(severity)
	{
		case LOG_INFO:
			_file << "[INFO " << t << "] - ";
			break;
		case LOG_WARNING:
			_file << "[WARNING " << t << "] - ";
			break;
		case LOG_ERROR:
			_file << "[ERROR " << t << "] - ";
			break;
	}
	return *this;
}

Logger::~Logger(void) 
{ 
	Lock l(Logger::s_LockEnableLogger);
	if(Logger::s_EnableLogger)
	{
		_file.close();
	}
}

void Logger::EnableLogger(bool enable)
{
	Lock l(Logger::s_LockEnableLogger);
	Logger::s_EnableLogger = enable;
}

void Logger::GetTime(char* t1)
{
	time_t t;
	time(&t);
	char* s = ctime(&t);
	//memcpy(t1, s, strlen(s) - 1);
	if (memcpy_s(t1, TIME_LENGTH, s, strnlen_s(s, TIME_LENGTH)))
	{
		printf("copy memory error");
	}
}
