/****************************************************************
Copyright  2011, Intel Corporation. All rights reserved.
*****************************************************************/
#include "SecString.h"
#include "StringUtils.h"

template <typename CTYPE>
SecString<CTYPE>::SecString(basic_string<CTYPE>& s) : _valid(false)
{
	try
	{
		if (!StrUtl::Encrypt(s, _s))
		{
			_valid = false;
			return;
		}
		sanitize(s);
		_valid = true;
	}
	catch(...)
	{
		//SetLastError(UNKNOWN_EXCEPTION);
		_valid = false;
	} 
}

template <typename CTYPE>
string SecString<CTYPE>::ToBase64String() const {
	if (_s.size() == 0)
		return string();
	TypeDefsNamespace::Base64 temp(((const unsigned char*)&_s[0]),_s.size());
	return temp.ToString();
}

template <typename CTYPE>
void SecString<CTYPE>::FromBase64String(const string& base) {
	TypeDefsNamespace::Base64 temp(base);
	//vector<unsigned char> tmp;
	temp.ToVector(_s);
}

template <typename CTYPE>
bool SecString<CTYPE>::decrypt(SanitizedStr<CTYPE>& outStr) const
{
	if (!isValid())
	{
		return false;
	}
	try
	{
		basic_string<CTYPE> s1;
		if (!StrUtl::Decrypt(_s, s1))
		{
			return false;
		}
		outStr = s1;
		sanitize(s1);
		return true;
	}
	catch (...)
	{
		return false;
	}
}

//
// Future: use memeset_s intead
//
// gcc #ifdef gcc and #pragma optimize("-no-dead-code-removal")

template <typename CTYPE>
void SecString<CTYPE>::sanitize(basic_string<CTYPE>& s)
{
	if (!s.empty())
		SecureZeroMemory(&s[0], s.capacity()*sizeof(CTYPE));
}

/*
template <typename CTYPE>
void SecString<CTYPE>::sanitize(vector<CTYPE>& v)
{
	if (!v.empty())
		SecureZeroMemory(&v[0], s.capacity()*sizeof(CTYPE));
}
*/
template <typename CTYPE>
void SecString<CTYPE>::sanitize(vector<unsigned char>& v)
{
	if (!v.empty())
		SecureZeroMemory(&v[0], v.capacity());
}
template class SecString<char>;
template class SecString<wchar_t>;
