//  Copyright (C) 2007 Intel Corporation
#include "SocketLib.h"
#include "SOCKS.h"
#include "stdio.h"


#define BACK_LOG_SIZE 4
#define MAX_BUF_SIZE 8192

const INT32 SOCKS_V5_TIMEOUT					= 60 * 1000; //timeout is in milliseconds

#ifndef _WIN32
#include <fcntl.h>
#include <unistd.h>
#else
#include "conio.h"
#endif

UINT32 CallbacksSocket::_conID = 0;


string Socket::GetErrorMessage(SOCKET_LIB_RESULT res)
{
	string msg = "";
	switch(res)
	{
		case SOCKET_LIB_RES_OK:
			msg = "Connection to Intel(r) AMT was succesfully opened";
			break;
		case SOCKET_LIB_RES_ERROR:
			msg = "can't open connection to Intel(r) AMT - reason general error";
			break;
		case SOCKET_LIB_RES_SOCKET_ERROR:
			msg = "can't open connection to Intel(r) AMT - reason general error";
			break;
		case SOCKET_LIB_RES_TLS_CONNECTION_FAILED:
			msg = "can't open connection to Intel(r) AMT - reason TLS error";
			break;
		case SOCKET_LIB_RES_SOCKS_ERROR:
			msg = "can't open connection to Intel(r) AMT - reason SOCKS error";
			break;
		case SOCKET_LIB_RES_SOCKS_AUTH_FAIL:
			msg = "can't open connection to Intel(r) AMT - reason SOCKS Auth fail";
			break;
	}
	return msg;
}

// if no param given this function will return the default IP stack supported
// otherwise this function will try to resolve the given address/hostname
// and return whether it mapped to associated with IPv4 address or IPv6 address
IP_VERSION IPSocket::SupportedIPStack(const char* hostname)
{
	IP_VERSION ver = IP_VERSION_NOT_SET;
	struct addrinfo *res, *aip;
    struct addrinfo hints;
    SOCKET sock = -1;
    int error;
    /* Get host address. Any type of address will do. */
	memset(&hints, 0, sizeof(hints));
    hints.ai_socktype = SOCK_STREAM;
	hints.ai_family = AF_UNSPEC;
    error = getaddrinfo(hostname, NULL, &hints, &res);
    if (error != 0) {
       return IP_VERSION_NOT_SET;
    }
    /* Try all returned addresses until one works */
    for (aip = res; aip != NULL; aip = aip->ai_next) {
       /*
       * Open socket. The address type depends on what
       * getaddrinfo() gave us.
       */
       sock = socket(aip->ai_family, aip->ai_socktype, aip->ai_protocol);
	   if (sock == INVALID_SOCKET) {
          freeaddrinfo(res);
		  continue;
       }
	   ver = (aip->ai_family == AF_INET6) ? IPv6 : IPv4;
#ifdef _WIN32
	closesocket(sock);
#else
	::close(sock);
#endif
	   freeaddrinfo(res);
	   break;
    }
	return ver;
}

IPSocket::IPSocket():
_socket(INVALID_SOCKET),
_port(0),
_isNonBlocking(false),
_ipVersion(IP_VERSION_NOT_SET), _tempReceiveBuffer(MAX_BUF_SIZE*2),_txSem()
{
}

IPSocket::IPSocket(const INT32& type, const INT32& protocol,  const UINT16& localPort, const string& localIP, IP_VERSION ipVersion):
_socket(INVALID_SOCKET),
_port(0),
_isNonBlocking(false),
_ipVersion(ipVersion), _tempReceiveBuffer(MAX_BUF_SIZE*2),_txSem()

{
	// determine if localIP is IPv4 address OR IPv6 address
	if(_ipVersion == IP_VERSION_NOT_SET)
		_ipVersion = IPSocket::SupportedIPStack(localIP.c_str());//determineIPVersion(localIP.c_str());

	int af = (_ipVersion == IPv4) ? AF_INET : AF_INET6;

	// open socket according to protocol and connection type
	if( INVALID_SOCKET == (_socket = socket(af, type, protocol)) ){
		perror("socket failed");
		exit(EXIT_FAILURE);
	}
	#ifndef _WIN32
	int enable =  1;
	int ret = setsockopt( _socket, SOL_SOCKET, SO_REUSEADDR, &enable, sizeof(enable) );
	#endif

	if(_ipVersion == IPv4)
	{
		struct sockaddr_in addrIn;
		memset(&addrIn, 0, sizeof(sockaddr_in));

		// fill sockaddr according to family type
		// IP family type (internet)
		addrIn.sin_family      = af;
		addrIn.sin_port        = htons(localPort);

		if( localIP.empty() ){
			addrIn.sin_addr.s_addr = INADDR_ANY;
		}
		else {
			addrIn.sin_addr.s_addr = inet_addr(localIP.c_str());
		}
		
		if (SOCKET_ERROR == bind(_socket,(struct sockaddr *) &addrIn, sizeof(addrIn))){
			close();
			return; 
		}
		
		//fill in the port
		int addrLen = sizeof(sockaddr_in);
		if( getsockname(_socket,(sockaddr*)&addrIn, (socklen_t *)&addrLen) != 0 ){
			return;
		}
		_port = ntohs(addrIn.sin_port);
	}
	else if(_ipVersion == IPv6)
	{
		struct sockaddr_in6 addrIn;
		int len = sizeof(addrIn);
		memset(&addrIn, 0, sizeof(sockaddr_in6));

		// fill sockaddr according to family type
		// IP family type (internet)
		addrIn.sin6_family      = AF_INET6;
		addrIn.sin6_port        = htons(localPort);
		if (localIP.empty())
		{
			addrIn.sin6_addr = in6addr_any;
		}
		else 
		{
#ifdef _WIN32
		WSAStringToAddress((LPSTR)localIP.c_str(), AF_INET6, NULL, (LPSOCKADDR) &addrIn, &len);
#else			
		inet_pton(AF_INET6, localIP.c_str(), &addrIn.sin6_addr.s6_addr);
#endif
		}

		if (SOCKET_ERROR == bind(_socket,(struct sockaddr *) &addrIn, sizeof(addrIn))){
				close();
				return; 
		}
		//fill in the port
		int addrLen = sizeof(sockaddr_in6);
		if( getsockname(_socket,(sockaddr*)&addrIn, (socklen_t *)&addrLen) != 0 ){
			return;
		}
		_port = ntohs(addrIn.sin6_port);
	}
}

IPSocket::IPSocket(SOCKET bindedSocket, IP_VERSION ipVersion): _tempReceiveBuffer(MAX_BUF_SIZE*2), 
_isNonBlocking(false), _port(0), _ipVersion(IP_VERSION_NOT_SET), _txSem()
{
	_socket = bindedSocket;
	// use large enough struct in order to suit both IPv4 & IPv6 address
	sockaddr_in6 addrIn;
	int lenAddr = sizeof(addrIn);
	// retrive local address of the socket
	if(getsockname((int)_socket, (struct sockaddr*)&addrIn, (socklen_t*)&lenAddr) != 0)
		return;
	
	_ipVersion = ipVersion;
	// fill in the port
	_port = ntohs(addrIn.sin6_port);
}

IPSocket::~IPSocket()
{
	close();
}

bool IPSocket::setRecvBuf(const INT32& recvBufSize)
{	
	if( setsockopt (_socket,  
					SOL_SOCKET,  
					SO_RCVBUF,  
					reinterpret_cast<const char*>(&recvBufSize),  
					sizeof(recvBufSize)) == SOCKET_ERROR) {
		return false;
	}
	return true;
}

bool IPSocket::setSendBuf(const INT32& sendBufSize)
{
	if (setsockopt (_socket,  
					SOL_SOCKET,  
					SO_SNDBUF,  
					reinterpret_cast<const char*>(&sendBufSize),  
					sizeof(sendBufSize)) == SOCKET_ERROR) {
		return false;
	}
	return true;
}

SOCKET_LIB_RESULT IPSocket::connect_v4(const string& ipAddr, const UINT16& port)
{
	//connect to server
	sockaddr_in servAddr;
	memset(&servAddr, 0, sizeof(sockaddr_in));

	servAddr.sin_family = AF_INET;
	unsigned long  addr = INADDR_NONE;
	addr = inet_addr(ipAddr.c_str());
	
	if( addr == INADDR_NONE || addr == INADDR_ANY ){
		struct addrinfo hints, *res = NULL;
		memset(&hints, 0, sizeof(hints)); 
		hints.ai_family = AF_INET; 
		hints.ai_socktype = SOCK_STREAM; 
		hints.ai_protocol = IPPROTO_TCP; 
		if( getaddrinfo( ipAddr.c_str(), NULL, &hints, &res ) ) 
		{ 
			Logger::GetInstance(Logger::LOG_WARNING) << "<IPSocket::connect_v4> getaddrinfo failed - please check IP/Host name" << endl;
			return SOCKET_LIB_RES_SOCKET_ERROR;
		} 
		struct addrinfo *addrInfo = res;
		while( addrInfo != 0){
			if( addrInfo->ai_family == AF_INET && addrInfo->ai_socktype == SOCK_STREAM 
				&&  addrInfo->ai_protocol == IPPROTO_TCP && 
				addrInfo->ai_addrlen == sizeof(sockaddr_in) ){
				servAddr = *((sockaddr_in*)(addrInfo->ai_addr));
				break;
			}
			addrInfo = addrInfo->ai_next;
		}
		if( res ){
			freeaddrinfo(res);
		}
		if( !addrInfo ){
			return SOCKET_LIB_RES_ERROR;
		}

	}
	else{
		servAddr.sin_addr.s_addr = addr;
	}

	servAddr.sin_port = htons(port);

	int res = ::connect(_socket, (struct sockaddr*)&servAddr, sizeof(servAddr));
	if( res == SOCKET_ERROR ) {
		Logger::GetInstance(Logger::LOG_WARNING) << "<IPSocket::connect_v4> Failed to establishe connection via socket" << endl;
		return SOCKET_LIB_RES_SOCKET_ERROR;
	}

	return SOCKET_LIB_RES_OK;
}
SOCKET_LIB_RESULT IPSocket::connect_v6(const string& ipAddr, const UINT16& port)
{
	//connect to server
	sockaddr_in6 servAddr;
	memset(&servAddr, 0, sizeof(sockaddr_in6));

	servAddr.sin6_family = AF_INET6;
	servAddr.sin6_addr = in6addr_any;

	if( ipAddr.empty() ){
			servAddr.sin6_addr = in6addr_any;
	}
	else {
		struct addrinfo hints, *res = NULL;
		memset(&hints, 0, sizeof(hints)); 
		hints.ai_family = AF_INET6; 
		hints.ai_socktype = SOCK_STREAM; 
		hints.ai_protocol = IPPROTO_TCP; 
		if( getaddrinfo( ipAddr.c_str(), NULL, &hints, &res ) ) 
		{ 
			Logger::GetInstance(Logger::LOG_WARNING) << "<IPSocket::connect_v6> getaddrinfo failed - please check IP/Host name" << endl;
			return SOCKET_LIB_RES_ERROR;
		} 
		struct addrinfo *addrInfo = res;
		while( addrInfo != 0){
			if( addrInfo->ai_family == AF_INET6 && addrInfo->ai_socktype == SOCK_STREAM 
				&&  addrInfo->ai_protocol == IPPROTO_TCP && 
				addrInfo->ai_addrlen == sizeof(sockaddr_in6) ){
				servAddr = *((sockaddr_in6*)(addrInfo->ai_addr));
				break;
			}
			addrInfo = addrInfo->ai_next;
		}
		if( res ){
			freeaddrinfo(res);
		}
		if( !addrInfo ){
			return SOCKET_LIB_RES_ERROR;
		}
	}

	servAddr.sin6_port = htons(port);

	int res = ::connect(_socket, (struct sockaddr*)&servAddr, sizeof(servAddr));
	if( res == SOCKET_ERROR ) {
		Logger::GetInstance(Logger::LOG_WARNING) << "<IPSocket::connect_v6> Failed to establishe connection via socket" << endl;
		return SOCKET_LIB_RES_ERROR;
	}

	return SOCKET_LIB_RES_OK;
}


SOCKET_LIB_RESULT IPSocket::connect(const string& ipAddr, const UINT16 port)
{
		if (_socket == INVALID_SOCKET) {
		// socket failed
			return SOCKET_LIB_RES_ERROR;
	}
	// invoke the correct connect function according to the ipVersion
	return (_ipVersion == IPv4) ? connect_v4(ipAddr, port) : connect_v6(ipAddr, port);
}

int IPSocket::prepareDataToSend(const BYTE*& data, int dataLen,const BYTE*& preparedData, int &envelopedDataLen)
{
	preparedData = data;
	envelopedDataLen = 0;
	return dataLen;
}

// WinSSLSocket (which implements a secure socket by SSPI(schannel) infrastructures)
// inherits from IPSocket class. 
// The IPSocket::send function is called from WinSSLSocket in both cases: 
// During the handshake process - to exchange keys.
// During the SOL or IDER sessions - to send the data that was encrypted by the key from the handshake.
INT32 IPSocket::send(const UINT8 * data, UINT32 dataLen)
{
	if (_socket == INVALID_SOCKET) {
		// socket failed
		return SOCKET_ERROR;
	}

	// We must acquire the tx semaphore, because we might
	// perform several calls to the socket's send in a loop
	// before we send out the entire message.
	Lock l(_txSem);

	int totalSent=0;
	const BYTE* preparedData = NULL;
	int dataToSendLen;
	bool isEncrypted = false;
	int envelopedDataLen;
	while(totalSent != dataLen) 
	{
		// prepareDataToSend virtual function is significant for 
		// WinSSLSocket which encrypts the data before sending it out.
		const BYTE* pDataToSend = data + totalSent;
		if(isEncrypted == false)
			dataToSendLen = prepareDataToSend(pDataToSend, dataLen - totalSent, preparedData, envelopedDataLen);

		// Failed to encrypt data
		if(envelopedDataLen == SOCKET_ERROR)
		{
			return SOCKET_ERROR;
		}

		int bytesSent = ::send(_socket,(char *)(preparedData), dataToSendLen + envelopedDataLen, 0);
		if (bytesSent ==0){
			return totalSent;
		}
		else if(bytesSent == SOCKET_ERROR){
#ifdef _WIN32
			int errcode = WSAGetLastError();
			if( errcode==WSAEWOULDBLOCK ){
				isEncrypted = true;
			
#else
			if ((errno == EWOULDBLOCK) || (errno == EAGAIN)) {
#endif
			}
			else{
				if( !totalSent ){
					totalSent = SOCKET_ERROR;
				}
				break;  // if total sent is less than necessary user will know of error
			}
		}
		else{ 
			isEncrypted = false;
			if(envelopedDataLen != 0)
				totalSent += dataToSendLen;
			else
				totalSent += bytesSent;
		}
    }
    return totalSent;
}

INT32 IPSocket::sendTo(const UINT8 * data, UINT32 dataLen, const string& ipAddr, const UINT16& port)
{
	if (_socket == INVALID_SOCKET || ipAddr.empty() ) {
		// socket failed
		return SOCKET_ERROR;
	}
	int bytesSent = 0;

	if(_ipVersion == IPv4)
	{
		struct sockaddr_in addrIn;
		memset(&addrIn, 0, sizeof(sockaddr_in));

		addrIn.sin_family      = AF_INET;
		unsigned long  addr = INADDR_NONE;
		addr = inet_addr(ipAddr.c_str());

		if( addr == INADDR_NONE || addr == INADDR_ANY ){
			struct addrinfo hints, *res = NULL;
			memset(&hints, 0, sizeof(hints)); 
			hints.ai_family = AF_INET; 
			hints.ai_socktype = SOCK_DGRAM; 
			if( getaddrinfo( ipAddr.c_str(), NULL, &hints, &res ) ) 
			{ 
				return SOCKET_ERROR;
			} 
			struct addrinfo *addrInfo = res;
			while( addrInfo != 0){
				if( addrInfo->ai_family == AF_INET && 
					addrInfo->ai_socktype == SOCK_DGRAM &&
					addrInfo->ai_addrlen == sizeof(sockaddr_in) ){

					addrIn = *((sockaddr_in*)(addrInfo->ai_addr));
					break;
				}
				addrInfo = addrInfo->ai_next;
			}
			if( res ){
				freeaddrinfo(res);
			}

			if( !addrInfo ){
				return SOCKET_ERROR;
			}

		}
		else{
			addrIn.sin_addr.s_addr = addr;
		}
		addrIn.sin_port        = htons(port);

		bytesSent = sendto(_socket,(char *)data, dataLen,0, (struct sockaddr*)&addrIn, sizeof(addrIn));
	}
	else if(_ipVersion == IPv6)
	{
		struct sockaddr_in6 addrIn;
		memset(&addrIn, 0, sizeof(sockaddr_in6));
		addrIn.sin6_family = AF_INET6;
		int len = sizeof(sockaddr_in6);
#ifdef _WIN32
		WSAStringToAddress((LPSTR)ipAddr.c_str(), AF_INET6, NULL, (LPSOCKADDR) &addrIn, &len);
#else			
		inet_pton(AF_INET6, ipAddr.c_str(), &addrIn.sin6_addr.s6_addr);
#endif
		addrIn.sin6_port = htons(port);
		bytesSent = sendto(_socket,(char *)data, dataLen,0, (struct sockaddr*)&addrIn, sizeof(addrIn));
	}
	
	if (bytesSent ==0)
	{
		return SOCKET_ERROR;
	}
	return bytesSent;
}

int IPSocket::prepareReceivedData(BYTE*& data, int bytesReceived)
{
	return bytesReceived;
}
INT32 IPSocket::receive(UINT8 *data, UINT32 dataLen, INT32 timeout)
{
	if (_socket == INVALID_SOCKET) {
		// socket failed
		return SOCKET_ERROR;
	}

	// Get the data was received at the previous call to receive function. 
	// When TCP connection is established, this function always returns zero,
	// when TLS-SSPI connection is established may return non-zero value. This happens because the packets that the FW sends us are too big,
	// and the data chouldn't be decrypted untill the whole packet is received, we must got a longer data 
	// than the dataLen value and save the extra data in an internal buffer to the next receive function call. 
	int reservedPlace = getInternalBUffer(data, dataLen);
	int freeSpace = dataLen - reservedPlace;
	if(freeSpace == 0)
		return reservedPlace;
	
	int maxBufSize = freeSpace;
	UINT8 * rcvBuf = data;
	// In case if TLS-SSPI, increase the length of the receive buffer.
	if(_isTLS())
	{
		if(maxBufSize >= MAX_BUF_SIZE)
			maxBufSize = MAX_BUF_SIZE*2;
		rcvBuf = &_tempReceiveBuffer[0];
	}
	
    
	int totalReceived = 0;
	

	// _reservedPlace() function returns the bytes length that couldn't be parsed in the previous IPSocket::receive() call.
	// When TCP connection is established, this function always returns zero,
	// when TLS-SSPI connection is established this function may return non-zero value. 
	while((totalReceived + _reservedPlace(dataLen)) < maxBufSize)
	{
		if (timeout != INFINITE) {
			fd_set fdset;
			FD_ZERO(&fdset);
			FD_SET(_socket, &fdset);

			struct timeval tv;
			tv.tv_sec = 0;
			tv.tv_usec = timeout * 1000; // in ms

			int status = select((int)_socket+1, &fdset, NULL, NULL, &tv);

			if (status!=1)
			{
				return 0;
			}
		}

		int bytesReceived = recv(_socket, reinterpret_cast<char*>(rcvBuf + totalReceived), 
						  maxBufSize-(totalReceived+_reservedPlace(dataLen)),0);
		
		if (bytesReceived == 0)
		{
			if( !totalReceived){
				totalReceived = SOCKET_ERROR;
			}
			break;
		}
		//if the socket would block then
		//examine the socket and continue when it will be available
		else if(bytesReceived == SOCKET_ERROR) 
		{
#ifdef _WIN32
			int errcode = WSAGetLastError();
			if( errcode==WSAEWOULDBLOCK ){
#else
			if ((errno == EWOULDBLOCK) || (errno == EAGAIN)) {
#endif
				if( _isNonBlocking ){
					break;
				}
			}
			if( !totalReceived){
				totalReceived = SOCKET_ERROR;
			}

			break;
		}
		else{

			// prepareReceivedData virtual function is significant for 
			// WinSSLSocket which dencrypts the data.
			BYTE* pData = rcvBuf + totalReceived;
			int dataReceivedLen = prepareReceivedData(pData, bytesReceived);
			if(SOCKET_ERROR == dataReceivedLen)
			{
				if( !totalReceived){
					totalReceived = SOCKET_ERROR;
				}
				break;
			}
			totalReceived += dataReceivedLen;

			// the data was received is longger than he freeSpace, stop to get more data.
			if(totalReceived > freeSpace)
				break;
			
			if(_readingCompleted() == true)
				break;
		}
	}
	
	
	if(totalReceived == SOCKET_ERROR)
	{
		return SOCKET_ERROR;
	}
	else
	{
		UINT8* pData = data+reservedPlace;
		reservedPlace += copyTempBuffer(pData, freeSpace, totalReceived);
		return reservedPlace;
	}
	
}

INT32 IPSocket::receiveFrom(UINT8 *data, UINT32 dataLen, string& ipAddr, UINT16& port)
{
	if (_socket == INVALID_SOCKET) {
		return SOCKET_ERROR;
	}
	int bytesReceived = 0;
	if(_ipVersion == IPv4)
	{
		struct sockaddr_in addrIn;
		memset(&addrIn, 0, sizeof(sockaddr_in));
		int addrLen = sizeof(addrIn);
		bytesReceived = recvfrom(_socket, (char*)data, dataLen, 0,
									(struct sockaddr*)&addrIn, (socklen_t*)&addrLen);
		if( bytesReceived > 0 ){
			port = ntohs(addrIn.sin_port);
			ipAddr = string(inet_ntoa(addrIn.sin_addr));
		}
		else{ 
	#ifdef _WIN32
			int errcode = WSAGetLastError();
			if( errcode==WSAEWOULDBLOCK ){
	#else
			if ((errno == EWOULDBLOCK) || (errno == EAGAIN)) {
	#endif
				if( _isNonBlocking ){
					bytesReceived = 0;
				}
			}	
		}
	}
	else if(_ipVersion == IPv6)
	{
		struct sockaddr_in6 addrIn;
		memset(&addrIn, 0, sizeof(sockaddr_in6));
		int addrLen = sizeof(addrIn);
		bytesReceived = recvfrom(_socket, (char*)data, dataLen, 0,
									(struct sockaddr*)&addrIn, (socklen_t*)&addrLen);
		if( bytesReceived > 0 ){
			port = ntohs(addrIn.sin6_port);
			char addressAsString[1000];
			memset(addressAsString, 0, sizeof(addressAsString));
	#ifdef _WIN32
			DWORD addressSize = sizeof(addressAsString) - 1;
			WSAAddressToString((LPSOCKADDR) &addrIn, addrLen, NULL, addressAsString, &addressSize);
	#else
			unsigned int addressSize = sizeof(addressAsString) - 1;
			inet_ntop(AF_INET6, &addrIn.sin6_addr, addressAsString, addressSize);
	#endif
			ipAddr = addressAsString;
		}
		else{ 
	#ifdef _WIN32
			int errcode = WSAGetLastError();
			if( errcode==WSAEWOULDBLOCK ){
	#else
			if ((errno == EWOULDBLOCK) || (errno == EAGAIN)) {
	#endif
				if( _isNonBlocking ){
					bytesReceived = 0;
				}
			}	
		}		
	}
	return bytesReceived;
}

INT32 IPSocket::close()
{
#ifdef _WIN32
	int res = closesocket(_socket);	
#else
	int res = ::close(_socket);
#endif
	_socket = INVALID_SOCKET;
	return res;
}

INT32 IPSocket::getType()
{
	return SOCK_STREAM;
}

SOCKET IPSocket::getSocket()
{
	return _socket;
}

UINT16 IPSocket::getPort()
{
	return _port;
}

bool IPSocket::setNonBlocking( bool enable )
{
#ifdef _WIN32
	ULONG param = (enable)?1:0;
	if( ioctlsocket(_socket, FIONBIO, &param) == SOCKET_ERROR ){
		return false;
	}
#else
	int flag = fcntl(_socket, F_GETFL);
	if (flag == -1) {
		return false;
	}

	if (enable) {

		flag |= (int)O_NONBLOCK;
	} else {
		flag &= (int)~O_NONBLOCK;
	}

	if (fcntl(_socket, F_SETFL, flag) == -1) {
		return false;
	}
#endif
	_isNonBlocking = enable;
	return true;
}

#ifdef _WIN32
bool IPSocket::setIPFragment( bool enable)
{
	int frag = (enable)?1:0;
	if( setsockopt(_socket, IPPROTO_IP, IP_DONTFRAGMENT, reinterpret_cast<const char*>(&frag), sizeof(frag)) == SOCKET_ERROR ){
		return false;
	}

	return true;
}
#endif

//////////////////////////////////////////////////////////////////////////////////////////
//// Server Socket classes
//////////////////////////////////////////////////////////////////////////////////////////
IPServerSocket::IPServerSocket(const INT32& type, const INT32& protocol,  const UINT16& localPort, const string& localIP, IP_VERSION ipVersion ):
IPSocket(type, protocol, localPort, localIP, ipVersion)
{
	if (listen(_socket, BACK_LOG_SIZE) == SOCKET_ERROR) 
	{
		close();
	}
}

Socket * IPServerSocket::accept()
{
	IPSocket *				sock = NULL;
	if (_socket == INVALID_SOCKET) {
		// socket failed
		return 0;
	}
	SOCKET sockFd = INVALID_SOCKET;
	int						addrLen;
	unsigned short tmpPort;
	if(_ipVersion == IPv4)
	{
		struct sockaddr_in		addrIn;
		addrLen = sizeof(struct sockaddr_in);
		sockFd = ::accept(_socket, (struct sockaddr*)&addrIn, (socklen_t *)&addrLen);
		tmpPort = addrIn.sin_port;
	}

	else if(_ipVersion == IPv6)
	{
		struct sockaddr_in6		addrIn;
		addrLen = sizeof(struct sockaddr_in6);
		sockFd = ::accept(_socket, (struct sockaddr*)&addrIn, (socklen_t *)&addrLen);
		tmpPort = addrIn.sin6_port;
	}
	else
	{
		// should never happen
		return 0;
	}

	if (sockFd != INVALID_SOCKET) 
	{
		sock = new IPSocket;
		if (sock){
			// conevrt the tmpPort to host order
			sock->_port = ntohs(tmpPort);
			sock->_socket = sockFd;
			sock->_ipVersion = _ipVersion;
		}
	}
	return sock;
}

///////////////////////////////////////////////////////////////////
// SSLSocket class
///////////////////////////////////////////////////////////////////
SSLSocket::SSLSocket(const UINT16 localPort, const string& localIP, IP_VERSION ipVersion):
TCPSocket(localPort, localIP, ipVersion)
{}


SSLSocket::SSLSocket(SOCKET bindedSocket, IP_VERSION ipVersion):
TCPSocket(bindedSocket, ipVersion)
{}

SSLSocket::~SSLSocket()
{
}

SPtr<Socket> SSLSocket::createSSLSocket(string caFile, string caPath, string privCert, string cert_pwd, string clientCert, IP_VERSION ipVersion, const string &cipher, bool enhancedTls, SOCKET bindedSocket)
{

#ifdef _WIN32
	if(bindedSocket == NULL)
		return SPtr<Socket>(new WinSSLSocket(0, "", ipVersion, clientCert, enhancedTls));
	else
		return SPtr<Socket>(new WinSSLSocket(bindedSocket, ipVersion, clientCert, enhancedTls));
#else
	if(bindedSocket == NULL)
		return SPtr<Socket>(new OpenSSLSocket(caFile, caPath, privCert, cert_pwd, ipVersion, true, cipher, 0, ""));
	else
		return SPtr<Socket>(new OpenSSLSocket(bindedSocket, caFile, caPath, privCert, cert_pwd, ipVersion, true, cipher));
#endif
}


///////////////////////////////////////////////////////////////////
// WinSSLSocket class
///////////////////////////////////////////////////////////////////
#ifdef _WIN32

// There are special requirements for the Intel AMT client certificates. The certificate needs to contain the following OIDs:
// 1.3.6.1.5.5.7.3.2 OID - marks the certificate as a TLS client certificate.
// 2.16.840.1.113741.1.2.1 OID - used by the Intel AMT device to authenticate the Redirection Library.
const LPSTR WinSSLSocket::_TLSClientOID = "2.16.840.1.113741.1.2.1";
const DWORD WinSSLSocket::_TLSClientOIDLen = strlen(_TLSClientOID);
const LPSTR WinSSLSocket::_AMTClientOID = "1.3.6.1.5.5.7.3.2";
const DWORD WinSSLSocket::_AMTClientOIDLen = strlen(_AMTClientOID);
bool WinSSLSocket::_isMutual = false;

WinSSLSocket::WinSSLSocket(const USHORT& localPort, const string& localIP, IP_VERSION ipVersion, string client_cert, bool enhancedTls) :
SSLSocket(localPort, localIP, ipVersion), _clientCerts(NULL), _inStream(MAX_BUF_SIZE*2), _outStream(NULL), _ipAddr(NULL), _inStreamIndex(0), 
	_connectionEstablished(false), _maxMsgLen(0), _clientCertName(client_cert),
	_internalBuffer(MAX_BUF_SIZE * 2), _internalBufferIndex(0), _alg(NULL)
{
	 _clientCertName = client_cert;
	 _encryptMessage = false;
	 _enhancedTls = enhancedTls;
	 memset(&_clientCred, 0, sizeof(SCHANNEL_CRED));
	 memset(&_credsHandle, 0, sizeof(CredHandle));	 
	 memset(&_ctxtHandle, 0, sizeof(CtxtHandle));
	 memset(&_msgSizes, 0, sizeof(SecPkgContext_StreamSizes));
}

WinSSLSocket::WinSSLSocket(SOCKET bindedSocket, IP_VERSION ipVersion, string client_cert, bool enhancedTls) :
		SSLSocket(bindedSocket, ipVersion), _clientCerts(NULL), _inStream(MAX_BUF_SIZE),_outStream(NULL), _ipAddr(NULL),_inStreamIndex(0), 
			_connectionEstablished(false), _maxMsgLen(0),  _clientCertName(client_cert),
			_internalBuffer(MAX_BUF_SIZE * 2), _internalBufferIndex(0), _alg(NULL)
{
	  _clientCertName = client_cert;
	  _encryptMessage = false;
	  _enhancedTls = enhancedTls;
	  memset(&_clientCred, 0, sizeof(SCHANNEL_CRED));
	  memset(&_credsHandle, 0, sizeof(CredHandle));
	  memset(&_ctxtHandle, 0, sizeof(CtxtHandle));
	  memset(&_msgSizes, 0, sizeof(SecPkgContext_StreamSizes));
}

WinSSLSocket::~WinSSLSocket()
{
	close();
}

SOCKET_LIB_RESULT WinSSLSocket::connect(const string& ipAddr, const UINT16 port)
{
	SOCKET_LIB_RESULT res; 

	if ((res = SSLSocket::connect(ipAddr, port)) != SOCKET_LIB_RES_OK)
		return res;

	if((res = handshake(ipAddr, port)) != SOCKET_LIB_RES_OK)
		return res;

	// _connectionEstablished variable indicates that TLS/SSL handshake was successfully completed;
	// a TLS/SSL connection has been established.
	_connectionEstablished = true;

	return SOCKET_LIB_RES_OK;	 
}
bool WinSSLSocket::getIsMutual()
{
	return _isMutual;
}

void WinSSLSocket::setIsMutual(bool isMutual)
{
	_isMutual = isMutual;
}

SOCKET_LIB_RESULT WinSSLSocket::_buildClientCredentials()
{
	ZeroMemory(&_clientCred, sizeof(_clientCred));
	
	_clientCred.dwVersion  = SCHANNEL_CRED_VERSION; 

	if(getIsMutual())
	{
		if(_getClientCerts() == 1)
		{
			_clientCred.cCreds     = 1;
			_clientCred.paCred     = &_clientCerts;
		}
	}

	// Bit flags for supported protocols TLS 1.0, TLS 1.1. TLS 1.2
	_clientCred.grbitEnabledProtocols = SP_PROT_TLS1;
	_clientCred.grbitEnabledProtocols |= SP_PROT_TLS1_1;
	_clientCred.grbitEnabledProtocols |= SP_PROT_TLS1_2;

	// Bit flags control the Schannel behavior.
	_clientCred.dwFlags |= SCH_CRED_AUTO_CRED_VALIDATION;
	_clientCred.dwFlags |= SCH_CRED_NO_DEFAULT_CREDS;

	// Supported ciphers algorithms
	_clientCred.cSupportedAlgs    = 0;
    _clientCred.palgSupportedAlgs = NULL;

	// Limit the possible cipher suites to 128 - WA to a FW performance bug in old AMT platforms.
	_clientCred.dwMaximumCipherStrength = _enhancedTls ? 256 : 128;
	_clientCred.dwMinimumCipherStrength = -1;
	

	// Create an SSPI credential.
    SECURITY_STATUS secRes = AcquireCredentialsHandle(NULL,
													 UNISP_NAME,		   // Schannel packet name
                                                     SECPKG_CRED_OUTBOUND, // Flags indicating use
													 NULL,                 // Reserved
													 &_clientCred,         
                                                     NULL,                 // Reserved
													 NULL,                 // Reserved
                                                     &_credsHandle,        // (out) Cred Handle
                                                     NULL);                // (out) Lifetime (optional)
	if(FAILED(secRes))
	{
		_printLogMsg("_buildClientCredentials", secRes);
		return SOCKET_LIB_RES_TLS_CONNECTION_FAILED;
	}

	return SOCKET_LIB_RES_OK;
}




int WinSSLSocket::_getClientCerts()
{
	HCERTSTORE  hSystemStore;
	bool found = false;
	_clientCerts = NULL;

	// Open the certificate store to be searched.
	if((hSystemStore = CertOpenSystemStore(0, "MY")) == NULL)
	{
		Logger::GetInstance() << "WinSSLSocket::_getClienCerts: cannot open system store." << ends;
		return 0;
	}

	// If _clientCertName equals "", first certificate with appropriate OIDs will be pulled.
	if(_clientCertName == "")
	{
		CERT_ENHKEY_USAGE certEnhkeyUsage;
		LPSTR usagesKey[2];

		usagesKey[0] = _TLSClientOID;
		usagesKey[1] = _AMTClientOID;
	
		certEnhkeyUsage.cUsageIdentifier = 2;
		certEnhkeyUsage.rgpszUsageIdentifier = usagesKey; 

		if((_clientCerts = CertFindCertificateInStore(hSystemStore, X509_ASN_ENCODING | PKCS_7_ASN_ENCODING, 0, CERT_FIND_ENHKEY_USAGE, &certEnhkeyUsage, NULL)) != NULL)
		{
			found = true;
		}
	}
	else
	{
		// Search the first client certificate that its subject is equal to _clientCertName and it has the required OIDs.
		DWORD enhkeyUsageSize = 0;
		bool OIDs[2] = {false, false};
		CERT_ENHKEY_USAGE* certEnhkeyUsage = NULL;
		while(!found)
		{    
			if((_clientCerts = CertFindCertificateInStore(hSystemStore, X509_ASN_ENCODING | PKCS_7_ASN_ENCODING, 0, CERT_FIND_SUBJECT_STR_A, _clientCertName.c_str(), _clientCerts)) == NULL)
				break;
			 
			// Get the OIDs array of the above _clientCerts certificate:
			// The length of this array is unknown, so the first call to CertGetEnhancedKeyUsage() returns the length, 
			// certEnhkeyUsage will be allocted as this length and the second call to CertGetEnhancedKeyUsage() will return the OIDs list.
			if(!CertGetEnhancedKeyUsage(_clientCerts, 0, NULL, &enhkeyUsageSize))
				break;

			certEnhkeyUsage = new CERT_ENHKEY_USAGE[enhkeyUsageSize];

			if(!CertGetEnhancedKeyUsage(_clientCerts, 0, certEnhkeyUsage, &enhkeyUsageSize))
			{
				delete [] certEnhkeyUsage;
				break;
			}

			int oidLen = 0;
			// Go over the OIDs list to check if it contains the required OIDs.
			for(DWORD i = 0; i < certEnhkeyUsage->cUsageIdentifier; i++)
			{
				oidLen =strlen(certEnhkeyUsage->rgpszUsageIdentifier[i]);
				if(memcmp(certEnhkeyUsage->rgpszUsageIdentifier[i],
					_TLSClientOID,
					oidLen > _TLSClientOIDLen ? _TLSClientOIDLen : oidLen))
				{
					OIDs[0] = true;
					continue;
				}

				if(memcmp(certEnhkeyUsage->rgpszUsageIdentifier[i],
					_AMTClientOID,
					oidLen > _TLSClientOIDLen ? _AMTClientOIDLen : oidLen))
				{
					OIDs[1] = true;
					continue;
				}
			}

			delete [] certEnhkeyUsage;
			certEnhkeyUsage = NULL;

			if(OIDs[0] && OIDs[1])
			{
				found = true;
			}
			else
			{
				OIDs[0] = false;
				OIDs[1] = false;
			}
		}
	}

	CertCloseStore(hSystemStore, CERT_CLOSE_STORE_CHECK_FLAG);

	if(found == false)
	{
		if(_clientCerts)
		{
			CertFreeCertificateContext(_clientCerts);
		}
		return 0;
	}
	return 1; // find one appropriate cetificate
}

SOCKET_LIB_RESULT WinSSLSocket::_getContextAttributes()
{
	// Get message size attributes
	if(QueryContextAttributes(&_ctxtHandle, SECPKG_ATTR_STREAM_SIZES, &_msgSizes) != SEC_E_OK)
		return SOCKET_LIB_RES_ERROR;
	
	// Allocate output buffer.
	if(_maxMsgLen != _msgSizes.cbHeader + _msgSizes.cbTrailer + _msgSizes.cbMaximumMessage)
	{
		_maxMsgLen = _msgSizes.cbHeader + _msgSizes.cbTrailer + _msgSizes.cbMaximumMessage;

		_outStream.resize(_maxMsgLen);
	}

	return SOCKET_LIB_RES_OK;
}

SOCKET_LIB_RESULT WinSSLSocket::handshake(const string& ipAddr, const UINT16 port)
{
	SOCKET_LIB_RESULT res;

	// Build client credentials
	if((res = _buildClientCredentials()) != SOCKET_LIB_RES_OK)
		return res;

	_ipAddr = strdup(ipAddr.c_str()); 

	// Handshake process
	if((res = _handshakeProcess()) != SOCKET_LIB_RES_OK)
		return res;

	// Get context attributes
	if((res = _getContextAttributes()) != SOCKET_LIB_RES_OK)
		return res;

	return SOCKET_LIB_RES_OK;
}

SOCKET_LIB_RESULT WinSSLSocket::_handshakeProcess()
{
	// Start handshake process:
	DWORD reqAttributes = ISC_REQ_SEQUENCE_DETECT | ISC_REQ_REPLAY_DETECT | ISC_REQ_CONFIDENTIALITY |
				    ISC_REQ_EXTENDED_ERROR | ISC_REQ_ALLOCATE_MEMORY | ISC_REQ_STREAM |
				    ISC_REQ_MUTUAL_AUTH;

	_encryptMessage = false;

	SECURITY_STATUS secRes = SEC_I_CONTINUE_NEEDED;
	SecBufferDesc recvMsg, sendMsg;
	SecBuffer recvBuffs[2], sendBuffs;
	bool newContext = true, waitForResponse = false;
	int recvDataLen = 0;
	BYTE recvData[MAX_BUF_SIZE]= {0};

	// Start handshake loop until SEC_E_OK is returned.
    while(secRes != SEC_E_OK) 
	{
		if(waitForResponse)
		{
			int bytesRecv = receive(recvData + recvDataLen, MAX_BUF_SIZE - recvDataLen, INFINITE);
			if(bytesRecv == 0 || bytesRecv == SOCKET_ERROR)
			{
				return SOCKET_LIB_RES_TLS_CONNECTION_FAILED;
			}
			recvDataLen += bytesRecv;
		}

		// Prepare received buffers
		recvBuffs[0].pvBuffer = recvData;
		recvBuffs[0].cbBuffer = recvDataLen;
		recvBuffs[0].BufferType = SECBUFFER_TOKEN;

		recvBuffs[1].pvBuffer = NULL;
		recvBuffs[1].cbBuffer = 0;
		recvBuffs[1].BufferType = SECBUFFER_EMPTY;

		recvMsg.cBuffers = 2;
		recvMsg.pBuffers = recvBuffs;
		recvMsg.ulVersion = SECBUFFER_VERSION;
		
		// Prepare buffers to send
		sendBuffs.pvBuffer   = NULL;
		sendBuffs.BufferType = SECBUFFER_TOKEN;
		sendBuffs.cbBuffer   = 0;

		sendMsg.cBuffers  = 1;
		sendMsg.pBuffers  = &sendBuffs;
		sendMsg.ulVersion = SECBUFFER_VERSION;

		DWORD retAttributes;
		secRes = InitializeSecurityContext(&_credsHandle, 
											newContext? NULL :  &_ctxtHandle ,
											_ipAddr ,
											reqAttributes,
											0,
											0, 
											newContext? NULL :  &recvMsg ,
											0,
											newContext?  &_ctxtHandle : NULL,
											&sendMsg,
											&retAttributes,
											NULL);
		if(secRes == SEC_I_INCOMPLETE_CREDENTIALS)
		{
			Logger::GetInstance() << "WinSSLSocket::_handshakeProcess: client authentication is required; Personal Store does not contain appropriate client certificate."<< secRes << endl;
			return SOCKET_LIB_RES_TLS_CONNECTION_FAILED;
		}
		if(FAILED(secRes) && secRes != SEC_E_INCOMPLETE_MESSAGE)
		{
			_printLogMsg("_handshakeProcess", secRes);
			return SOCKET_LIB_RES_TLS_CONNECTION_FAILED;
		}
		
		if(newContext == true)
		{
			if(!(retAttributes | ISC_REQ_CONFIDENTIALITY))
				return SOCKET_LIB_RES_TLS_CONNECTION_FAILED;
			newContext = false;
		}

		// The Message from the Intel AMT has not completed - continue to read. 
		if(secRes == SEC_E_INCOMPLETE_MESSAGE)
		{
			waitForResponse = true;
			continue;
		}

		// Send the output buffer.
		if(0 != sendBuffs.cbBuffer && NULL != sendBuffs.pvBuffer)
		{
			int totalSent = send(static_cast<const BYTE *>(sendBuffs.pvBuffer), sendBuffs.cbBuffer);
			FreeContextBuffer(sendBuffs.pvBuffer);
			sendBuffs.pvBuffer = NULL;
			if(totalSent == 0 || totalSent == SOCKET_ERROR)
			{
				return SOCKET_LIB_RES_TLS_CONNECTION_FAILED;
			}
		}

		// Copy any leftover data from the "extra" buffer, and call again to InitializeSecurityContext function.
		if (SECBUFFER_EXTRA == recvBuffs[1].BufferType)
        {
			memmove_s(recvData, recvDataLen, recvData + (recvDataLen - recvBuffs[1].cbBuffer), recvBuffs[1].cbBuffer);
			recvDataLen = recvBuffs[1].cbBuffer;
			waitForResponse = false;
        }
        else
		{
			recvDataLen = 0;
			waitForResponse = true;
		}
	}

	_encryptMessage = true;

	return SOCKET_LIB_RES_OK;
}

int WinSSLSocket::close()
{
	if(_connectionEstablished == true)
	{
		// Prepare and send "shutdown" notification to schannel security context
		DWORD closeFlags = SCHANNEL_SHUTDOWN;

		SecBuffer closeBuffers;
		closeBuffers.cbBuffer = sizeof(closeFlags);
		closeBuffers.pvBuffer = &closeFlags;
		closeBuffers.BufferType = SECBUFFER_TOKEN;

		SecBufferDesc closeRequest;
		closeRequest.cBuffers = 1;
		closeRequest.pBuffers = &closeBuffers;
		closeRequest.ulVersion = SECBUFFER_VERSION; 
		
		
		if(FAILED(ApplyControlToken(&_ctxtHandle, &closeRequest)))
		{
			Logger::GetInstance() << "WinSSLSocket::close: failed to disconnect" << endl;
		}
		else
		{
		
			closeBuffers.cbBuffer = NULL;
			closeBuffers.pvBuffer = NULL;
			closeBuffers.BufferType = SECBUFFER_TOKEN;
		
			closeRequest.cBuffers = 1;
			closeRequest.pBuffers = &closeBuffers;
			closeRequest.ulVersion = SECBUFFER_VERSION;

			closeFlags = ISC_REQ_REPLAY_DETECT  | ISC_REQ_SEQUENCE_DETECT | ISC_REQ_CONFIDENTIALITY   |
						 ISC_REQ_EXTENDED_ERROR | ISC_REQ_ALLOCATE_MEMORY | ISC_REQ_STREAM;

			DWORD ctxtAttribute;
			SECURITY_STATUS secRes = InitializeSecurityContext(&_credsHandle, &_ctxtHandle, _ipAddr, closeFlags, 0,  SECURITY_NATIVE_DREP, NULL, 0, &_ctxtHandle, &closeRequest, &ctxtAttribute, NULL);

			if(SUCCEEDED(secRes))
			{
				_encryptMessage = false;
				int bytesSent = IPSocket::send(static_cast<const BYTE *>(closeBuffers.pvBuffer), closeBuffers.cbBuffer);
				if(bytesSent <= 0)
				{
					Logger::GetInstance() << "WinSSLSocket::close: failed to send close notification" << endl;
				}
			}
			else
			{
				_printLogMsg("close", secRes);
			}
			FreeContextBuffer(closeBuffers.pvBuffer);
		}
	}

	// Free SSPI resources
	if(_clientCerts)
	{
		CertFreeCertificateContext(_clientCerts);
		_clientCerts = NULL;
	}

	DeleteSecurityContext(&_ctxtHandle);
	FreeCredentialsHandle(&_credsHandle);

	_connectionEstablished = false;
	free(_ipAddr);
	_ipAddr = NULL;

	// Call IPSocket close function.
	return IPSocket::close();
}

int WinSSLSocket::prepareDataToSend(const BYTE*& data, int dataLen, const BYTE*& preparedData, int &envelopedDataLen)
{
	if(_encryptMessage == false)
	{
		//The _encryptMessage is equal to false during the handshake process - the message shouldn't be encrypted.
		return TCPSocket::prepareDataToSend(data, dataLen, preparedData, envelopedDataLen);
	}
	else 
	{
     	_getContextAttributes();
		int packetLength =  0;	
		if(_msgSizes.cbMaximumMessage < dataLen)
			packetLength = _msgSizes.cbMaximumMessage;
		else
			packetLength = dataLen;
		
		std::copy(data, data+packetLength, _outStream.begin() + _msgSizes.cbHeader);

		SecBuffer dataBuffers[4];
		// Set header buffer
		dataBuffers[0].BufferType = SECBUFFER_STREAM_HEADER;
		dataBuffers[0].cbBuffer   = _msgSizes.cbHeader;
		dataBuffers[0].pvBuffer   = &_outStream[0];

		// Set message buffer
		dataBuffers[1].BufferType = SECBUFFER_DATA;
		dataBuffers[1].cbBuffer   = packetLength;
		dataBuffers[1].pvBuffer   = &_outStream[_msgSizes.cbHeader];

		// Set trailer buffer
		dataBuffers[2].BufferType = SECBUFFER_STREAM_TRAILER;
		dataBuffers[2].cbBuffer   = _msgSizes.cbTrailer;
		dataBuffers[2].pvBuffer   = &_outStream[_msgSizes.cbHeader + packetLength];

		dataBuffers[3].BufferType = SECBUFFER_EMPTY;

		SecBufferDesc dataPacket;
		dataPacket.ulVersion      = SECBUFFER_VERSION; 
		dataPacket.cBuffers       = 4;                 
		dataPacket.pBuffers       = dataBuffers;       

		// Encrypt message
		SECURITY_STATUS secRes = EncryptMessage(&_ctxtHandle, 0, &dataPacket, 0);
		if(FAILED(secRes))
		{
			_printLogMsg("prepareDataToSend", secRes);
			packetLength = SOCKET_ERROR;
		}

		envelopedDataLen = dataBuffers[0].cbBuffer + dataBuffers[2].cbBuffer; // return the unnecessary data length i.e. the header and the trailer sizes.
		preparedData = &_outStream[0];
		return packetLength; 
	}
}
int WinSSLSocket::_reservedPlace(int dataLen)
{
	if(dataLen >= MAX_BUF_SIZE)	
	// _inStreamIndex saves how many bytes of the data that couldn't be decrypted 
	// in previous WinSSLSocket::prepareReceivedData() call 
	// because the data wasn't fully received from the network.
		return _inStreamIndex;
	else
	// if the dataLen smaller than MAX_BUF_SIZE, it is in the athentication process,
	// the dataLen is too small and it's unnecessary to take in account the place of the _inStreamIndex.
		return 0;
}

INT32 WinSSLSocket::getInternalBUffer(UINT8 *&data, UINT32 dataLen)
{
	int freeSpace = dataLen;
	int reservedPlace = 0;
	if(_internalBufferIndex > 0)
	{
		if(_internalBufferIndex >= freeSpace)
		{
			std::copy(_internalBuffer.begin(), _internalBuffer.begin() + freeSpace, data);
			std::copy(_internalBuffer.begin() + freeSpace, _internalBuffer.begin() + _internalBufferIndex, _internalBuffer.begin());
			_internalBufferIndex -= freeSpace;
			reservedPlace += freeSpace;
			freeSpace = 0;
			return dataLen;
		}
		else
		{
			std::copy( _internalBuffer.begin(), _internalBuffer.begin() + _internalBufferIndex, data);
			freeSpace -= _internalBufferIndex;
			reservedPlace += _internalBufferIndex;
			_internalBufferIndex = 0;
			return reservedPlace;
		}
	}
	else
	{
		return 0;
	}
}

INT32 WinSSLSocket::copyTempBuffer(UINT8 *&data, UINT32 dataLen, int totalReceived)
{
	if(totalReceived >= dataLen)
	{
		std::copy(_tempReceiveBuffer.begin(), _tempReceiveBuffer.begin() + dataLen, data);
		std::copy(_tempReceiveBuffer.begin() + dataLen, _tempReceiveBuffer.begin() + totalReceived, _internalBuffer.begin() + _internalBufferIndex);
		_internalBufferIndex += totalReceived - dataLen;
		return dataLen;
		
	}
	else 
	{
		std::copy(_tempReceiveBuffer.begin(), _tempReceiveBuffer.begin() + totalReceived, data);
		_internalBufferIndex = 0;
		return totalReceived;
	}
}

 //data parameter is a pointer to the last received packet!
int WinSSLSocket::prepareReceivedData(BYTE*& data, int bytesReceived)
{
	if(_encryptMessage == false)
	{
		//The _encryptMessage is equal to false during the handshake process - the message shouldn't be decrypted.
		return TCPSocket::prepareReceivedData(data, bytesReceived);
	}
	else
	{
		bool doHandshake = false;
		int decryptedMsgLen = 0;
		// If more than one thread is decrypting, 
		// each thread should obtain a unique context.
		// critical section
		{
			Lock l(_decSem);
			if(bytesReceived < 0)
				return 0;

			if(_inStream.size() < (_inStreamIndex + bytesReceived))
				_inStream.resize(_inStreamIndex + bytesReceived);

			std::copy(data, data + bytesReceived, _inStream.begin() +_inStreamIndex);
		
			// In order to decrypt the data that was received, 
			// set dataPacket buffer with the recevied bytes and send to DecryptMessage function.
			SecBuffer dataBuffers[4];
			dataBuffers[0].BufferType = SECBUFFER_DATA;
			dataBuffers[0].cbBuffer  = bytesReceived + _inStreamIndex;
			dataBuffers[0].pvBuffer = &_inStream[0];

			SecBufferDesc dataPacket;
			bool endDecrypt = false; 

			// pIncompletedData keeps a pointer to the data that cannot be decrypted because the data isn't fully received from the network - 
			// this pointer is re-set in the beginning of every DecryptMessage method.
			// pIncompletedData and incompletedDataLen come into use when SEC_E_INCOMPLETE_MESSAGE is returned from DecryptedMessage function
			BYTE* pIncompletedData = &_inStream[0];
			int incompletedDataLen = _inStreamIndex + bytesReceived;


			while(!endDecrypt)
			{

				dataBuffers[1].BufferType = SECBUFFER_EMPTY;
				dataBuffers[2].BufferType = SECBUFFER_EMPTY;
				dataBuffers[3].BufferType = SECBUFFER_EMPTY;
		
     			dataPacket.ulVersion = SECBUFFER_VERSION;
				dataPacket.cBuffers = 4;
				dataPacket.pBuffers = dataBuffers;

				SECURITY_STATUS secRes = DecryptMessage(&_ctxtHandle, &dataPacket, 0, NULL);
			

				if(SEC_E_OK != secRes && 
					SEC_I_RENEGOTIATE != secRes && 
					SEC_E_INCOMPLETE_MESSAGE != secRes)
				{ 
					_printLogMsg("prepareReceivedData", secRes);
					return SOCKET_ERROR;
				}
			
				if(SEC_I_RENEGOTIATE == secRes)
				{
					// The call to handshake process will be outside the critical section.
					// Because the _handshakeProcess() function calls to IPSocket::send() and IPSocket::receive() functions
					// which call to preparedDataToSend and preparedReceivedData functions. 
					doHandshake = true;
				}

				// The data in the input buffer is incomplete. 
				// The application needs to read more data from 
				// the server and call again to DecryptMessage.
				if(SEC_E_INCOMPLETE_MESSAGE == secRes)
				{
					if(_inStream.size() < (incompletedDataLen))
						_inStream.resize(incompletedDataLen);

					std::copy(pIncompletedData, pIncompletedData + incompletedDataLen, _inStream.begin());  
					_inStreamIndex = incompletedDataLen; 
					return decryptedMsgLen;
				}

				int extraIndex = -1;
				for(int i = 0; i < 4; i++)
				{
					if(SECBUFFER_DATA == dataBuffers[i].BufferType)
					{
						std::copy((BYTE *)dataBuffers[i].pvBuffer, (BYTE *)dataBuffers[i].pvBuffer + dataBuffers[i].cbBuffer, data+decryptedMsgLen);
						decryptedMsgLen += dataBuffers[i].cbBuffer;	
					}
					else if(SECBUFFER_EXTRA == dataBuffers[i].BufferType)
					{ 
						// Just set an index to extra data location.
						extraIndex = i;
					}
				}

				// There is extra data, re-call to DecryptMessage function again.
				if(-1 !=  extraIndex)
				{
					dataBuffers[0].BufferType = SECBUFFER_DATA;
					dataBuffers[0].cbBuffer  = dataBuffers[extraIndex].cbBuffer;
					dataBuffers[0].pvBuffer = dataBuffers[extraIndex].pvBuffer;
					pIncompletedData = (BYTE*)dataBuffers[0].pvBuffer;
					incompletedDataLen =  dataBuffers[extraIndex].cbBuffer;
				}
				else 
				{
					endDecrypt = true;
				}
			}

			_inStreamIndex = 0;
		}

		if(doHandshake)
		{
			if(_handshakeProcess() != SOCKET_LIB_RES_OK)
			{
				Logger::GetInstance() << "WinSSLSocket::prepareReceivedData: cannot renegotiate connection" << endl;
				return -1;
			}
		}


		return decryptedMsgLen;
	}

}

bool WinSSLSocket::_readingCompleted()
{
	// if the _encrypteMessage is false, 
	// the handshake loop is in process, and we want to read just once.
	return _encryptMessage == false ? true : false;
}

void WinSSLSocket::_printLogMsg(string functionName, SECURITY_STATUS errCode)
{
	switch(errCode)
	{
	// A context handle that is not valid was specified in the phContext parameter.
	case SEC_E_INVALID_HANDLE:
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_INVALID_HANDLE" << ends;
		break;

	// The buffers are of the wrong type or no buffer of type SECBUFFER_DATA was found.
	case SEC_E_INVALID_TOKEN: 
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_INVALID_TOKEN" << ends;
		break;

		// The message has been altered
	case SEC_E_MESSAGE_ALTERED :
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_MESSAGE_ALTERED" << ends;
		break;

 		// The message was not received in the correct sequence.
	case SEC_E_OUT_OF_SEQUENCE: 
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_OUT_OF_SEQUENCE" << ends;
		break;

		//The message sender has finished using the connection and has initiated a shutdown 
	case SEC_I_CONTEXT_EXPIRED:
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_I_CONTEXT_EXPIRED" << ends;
		break;

		//The output buffer is too small.
	case SEC_E_BUFFER_TOO_SMALL:
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_BUFFER_TOO_SMALL" << ends;
		break;

		//The cipher chosen for the security context is not supported.
	case SEC_E_CRYPTO_SYSTEM_INVALID:
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_CRYPTO_SYSTEM_INVALID" << ends;
		break;

		//There is not enough memory available to complete the requested action.
	case SEC_E_INSUFFICIENT_MEMORY:
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_INSUFFICIENT_MEMORY" << ends;
		break;

		//Neither confidentiality nor integrity are supported by the security context.
	case SEC_E_QOP_NOT_SUPPORTED:
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_QOP_NOT_SUPPORTED" << ends;
		break;

		//An error occurred that did not map to an SSPI error code.
	case SEC_E_INTERNAL_ERROR: 
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_INTERNAL_ERROR" << ends;
		break;

		//The logon failed.
	case SEC_E_LOGON_DENIED:
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_LOGON_DENIED" << ends;
		break;

		//No authority could be contacted for authentication. The domain name of the authenticating party could be wrong, 
		//the domain could be unreachable, or there might have been a trust relationship failure.
	case SEC_E_NO_AUTHENTICATING_AUTHORITY:
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_NO_AUTHENTICATING_AUTHORITY" << ends;
		break;

		//No credentials are available in the security package.
	case SEC_E_NO_CREDENTIALS:
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_NO_CREDENTIALS" << ends;
		break;

		//The target was not recognized.
	case SEC_E_TARGET_UNKNOWN:
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_TARGET_UNKNOWN" << ends;
		break;

		//A context attribute flag that is not valid (ISC_REQ_DELEGATE or ISC_REQ_PROMPT_FOR_CREDS) was specified in the fContextReq parameter.
	case SEC_E_UNSUPPORTED_FUNCTION :
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_UNSUPPORTED_FUNCTION" << ends;
		break;

		// The principal that received the authentication request is not the same as the one passed into the pszTargetName parameter. This indicates a failure in mutual authentication
	case SEC_E_WRONG_PRINCIPAL: 
		Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_WRONG_PRINCIPAL" << ends;
		break;

		//The caller of the function does not have the necessary credentials.
	case SEC_E_NOT_OWNER:
			Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_NOT_OWNER" << ends;
		break;

		// The requested security package does not exist.
	case SEC_E_SECPKG_NOT_FOUND:
			Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_SECPKG_NOT_FOUND" << ends;
		break;

		//The credentials supplied to the package were not recognized.
	case SEC_E_UNKNOWN_CREDENTIALS:
			Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": SEC_E_UNKNOWN_CREDENTIALS" << ends;
		break;

		default:
			Logger::GetInstance() << "WinSSLSocket::"<< functionName << ": unknown error" << ends;
		break;
	}
				
}

#else
#include "openssl/x509v3.h"
#include "openssl/asn1.h"


///////////////////////////////////////////////////////////////////
// OpenSSL SharedLocks Socket class
///////////////////////////////////////////////////////////////////

OpenSSLSocket::SharedLocks OpenSSLSocket::_sharedLocks;
Semaphore* OpenSSLSocket::SharedLocks::_sharedLocks = NULL;

OpenSSLSocket::SharedLocks::SharedLocks()
{
	
}

void OpenSSLSocket::SharedLocks::Init()
{
	int num_locks = CRYPTO_num_locks();
	_sharedLocks = new Semaphore[num_locks];
	CRYPTO_set_locking_callback(_locking_callback);
	SSL_library_init();
	SSL_load_error_strings();
}

OpenSSLSocket::SharedLocks::~SharedLocks()
{
	if(_sharedLocks)
	{
		ERR_free_strings();
		EVP_cleanup();
		CRYPTO_cleanup_all_ex_data();
		delete [] _sharedLocks;
		_sharedLocks = NULL;
	}
}
////////////////////////////////////////////////////////////////////////////////////////////
////// OpenSSLSocket Socket class
////////////////////////////////////////////////////////////////////////////////////////////
OpenSSLSocket::OpenSSLSocket(string ca_file, string ca_path, string priv_cert, string cert_pass, IP_VERSION ipVersion, const bool tls, const string &cipher, const UINT16& localPort, const string& localIP):
SSLSocket(localPort, localIP, ipVersion)
{
	_ctx = NULL;
	_con = NULL;
	_tls = tls;
	//Remove AES256 as a WA to a FW performance bug.
	_cipher = cipher.empty() ? "ALL:eNULL:-AES256-SHA" : cipher;
	_caFile = ca_file;
	_caPath = ca_path;
	_privCert = priv_cert;
	_certPass = cert_pass;
	_sharedLocks.Init();
	_peerCert = NULL;
}

OpenSSLSocket::OpenSSLSocket(SOCKET bindedSocket, string ca_file, string ca_path, string priv_cert, string cert_pass, IP_VERSION ipVersion, const bool tls, const string &cipher):
SSLSocket(bindedSocket, ipVersion)
{
	_ctx = NULL;
	_con = NULL;
	_tls = tls;
	//Remove AES256 as a WA to a FW performance bug.
	_cipher = cipher.empty() ? "ALL:eNULL:-AES256-SHA" : cipher;
	_caFile = ca_file;
	_caPath = ca_path;
	_privCert = priv_cert;
	_certPass = cert_pass;
	_sharedLocks.Init();
	_peerCert = NULL;
}

OpenSSLSocket::~OpenSSLSocket()
{
	close();
}

void OpenSSLSocket::_locking_callback(INT32 mode, INT32 n, const char *file, INT32 line)
{
	if (mode & CRYPTO_LOCK) {
		_sharedLocks[n].acquire();
	} else {
		_sharedLocks[n].release();
	}
}

INT32 OpenSSLSocket::_passwd_callback(char *buf, INT32 size, INT32 rwflag, void *userdata)
{
	strncpy(buf, reinterpret_cast<const char*>(userdata), size);
	buf[size - 1] = '\0';
	return static_cast<INT32>(strlen(buf));
}

SOCKET_LIB_RESULT OpenSSLSocket::_verifyCert(string ipAddr)
{
	SOCKET_LIB_RESULT res = SOCKET_LIB_RES_TLS_CONNECTION_FAILED;

	X509_NAME *subj;
	int idxCN;

	// get peer certificate
	_peerCert = SSL_get_peer_certificate(_con);
	if (_peerCert == NULL) {
		goto err;
	}

	// get peer certificate's subject
	subj = X509_get_subject_name(_peerCert);
	if (subj == NULL) {
		goto err;
	}

	// go over all CN fields of the peer certificate's subject
	idxCN = -1;
	for (;;) {
		idxCN = X509_NAME_get_index_by_NID(subj, NID_commonName, idxCN);
		if (idxCN == -1) {
			break;
		}

		X509_NAME_ENTRY *ne = X509_NAME_get_entry(subj, idxCN);
		ASN1_STRING *a = X509_NAME_ENTRY_get_data(ne);

#ifdef _WIN32
		if (_stricmp(ipAddr.c_str(), (char *)a->data) == 0) {
#else
		if (strcasecmp(ipAddr.c_str(), (char *)a->data) == 0) {
#endif
			res = SOCKET_LIB_RES_OK;
			break;
		}
	}

err:
	return res;
}

SOCKET_LIB_RESULT OpenSSLSocket::connect(const string& ipAddr, const UINT16 port)
{
	if (_ctx || _con) {
		close();
	}

	SOCKET_LIB_RESULT res = TCPSocket::connect(ipAddr, port);
	if (res != SOCKET_LIB_RES_OK) {
		return res;
	}

	return handshake(ipAddr, port);
}

SOCKET_LIB_RESULT OpenSSLSocket::handshake(const string& ipAddr, const UINT16 port)
{
	char *ca_file = NULL;
	char *ca_path = NULL;

	SOCKET_LIB_RESULT res = SOCKET_LIB_RES_SOCKET_ERROR;

	if (_tls) {
		_ctx = SSL_CTX_new( TLSv1_client_method() );
	} else {
		_ctx = SSL_CTX_new( SSLv3_client_method() );
	}

	if (!_ctx) {
		goto err;
	}

	// Set the ctx to auto-retry session negotiation
	// during read/write if needed.

	if (_caFile.c_str() && (_caFile.length() > 0)) {
		ca_file = (char *)_caFile.c_str();
	}
	if (_caPath.c_str() && (_caPath.length() > 0)) {
		ca_path = (char *)_caPath.c_str();
	}

	SSL_CTX_load_verify_locations(_ctx, ca_file, ca_path);
	SSL_CTX_set_mode(_ctx, SSL_MODE_AUTO_RETRY | SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
	SSL_CTX_set_verify(_ctx, SSL_VERIFY_PEER, NULL);
	
	// add support for SHA2 verification
	EVP_add_digest(EVP_sha224());
	EVP_add_digest(EVP_sha256());
	EVP_add_digest(EVP_sha384());
	EVP_add_digest(EVP_sha512());

	if (_privCert.c_str() && (_privCert.length() > 0)) {
		if (!SSL_CTX_use_certificate_chain_file(_ctx, _privCert.c_str())) {
			// "SSLSocket::connect: failed to set certificate chain file file"
			res = SOCKET_LIB_RES_TLS_CONNECTION_FAILED;
			goto err;
		}

		// We must set the passward callback, even if we don't have a cert password.
		// This is because if the private key file is password protected but we don't
		// have the password for some reason, SSL_CTX_use_PrivateKey_file() will hang.
			SSL_CTX_set_default_passwd_cb_userdata(_ctx, (void *)_certPass.c_str());
			SSL_CTX_set_default_passwd_cb(_ctx, _passwd_callback);

		if (!SSL_CTX_use_PrivateKey_file(_ctx, _privCert.c_str(), SSL_FILETYPE_PEM)) {
			// "SSLSocket::connect: failed to parse private key file"
			res = SOCKET_LIB_RES_TLS_CONNECTION_FAILED;
			goto err;
		}
	}


	if (!SSL_CTX_set_cipher_list(_ctx, _cipher.c_str())) {
		goto err;
	}

	_con = SSL_new(_ctx);
	if (!_con) {
		goto err;
	}

	if (!SSL_set_fd(_con, static_cast<INT32>(_socket))) {
		goto err;
	}

	if (SSL_connect(_con) <= 0) {
		res = SOCKET_LIB_RES_TLS_CONNECTION_FAILED;
		goto err;
	}

	res = _verifyCert(ipAddr);
	if (res != SOCKET_LIB_RES_OK) {
		goto err;
	}
	
	return SOCKET_LIB_RES_OK;

err:
	{
		int error = ERR_get_error();
		if (error) {
			string pf = ERR_func_error_string(error);
			string pr = ERR_reason_error_string(error);
		}
	}
	close();
	return res;
}

INT32 OpenSSLSocket::send(const UINT8 * data, UINT32 dataLen)
{
	UINT32 totalSent = 0;
	int res;
	int saved_errno;

	if (!_con) {
		return SOCKET_ERROR;
	}

	// We must acquire the tx semaphore, because we might
	// perform several calls to the socket's send in a loop
	// before we send out the entire message.
	Lock l(_txSem);

	while (totalSent < dataLen) {
		res = SSL_write(_con, data, (dataLen - totalSent));
		saved_errno = errno;

		switch (SSL_get_error(_con, res)) {
		case SSL_ERROR_NONE:
			data += res;
			totalSent += res;
			break;

		case SSL_ERROR_WANT_WRITE:
		case SSL_ERROR_WANT_READ:
			// Something is blocking the operation
			// The docs say we should retry the operation.
			break;

		case SSL_ERROR_ZERO_RETURN:
			// Peer has closed the session
			return SOCKET_ERROR;
		case SSL_ERROR_WANT_CONNECT:
			return SOCKET_ERROR;
		case SSL_ERROR_WANT_ACCEPT:
			return SOCKET_ERROR;
		case SSL_ERROR_WANT_X509_LOOKUP:
			return SOCKET_ERROR;
		case SSL_ERROR_SYSCALL: {

			string pe = strerror(saved_errno);

			int error = ERR_get_error();
			if (error) {
				string pf = ERR_func_error_string(error);
				string pr = ERR_reason_error_string(error);
			} else {

			}

			return SOCKET_ERROR;
		}
		case SSL_ERROR_SSL: {

			string pe = strerror(saved_errno);

			int error = ERR_get_error();
			if (error) {
				string pf = ERR_func_error_string(error);
				string pr = ERR_reason_error_string(error);

			} else {
			}

			return SOCKET_ERROR;
		}
		default:
			return SOCKET_ERROR;
		}
	}

	return totalSent;
}

INT32 OpenSSLSocket::receive(UINT8 *data, UINT32 dataLen, INT32 timeout)
{
	UINT32 totalReceived = 0;
	int res;
	int saved_errno;

	if (!_con) {
		return SOCKET_ERROR;
	}

	Lock l(_txSem);

	while (totalReceived < dataLen) {
		if (!SSL_pending(_con)) {
			if (timeout != INFINITE) {
				fd_set fdset;
				struct timeval tv;
				int status;


				FD_ZERO(&fdset);
				FD_SET(_socket, &fdset);

				tv.tv_sec = 0;
				tv.tv_usec = timeout * 1000; // in ms

				status = select((int)_socket+1, &fdset, NULL, NULL, &tv);

				if (status != 1) {
					break;
				}
			}
		}

		res = SSL_read(_con, data, (dataLen - totalReceived));
		saved_errno = errno;

		switch (SSL_get_error(_con, res)) {
		case SSL_ERROR_NONE:
			data += res;
			totalReceived += res;
			break;

		case SSL_ERROR_WANT_WRITE:
		case SSL_ERROR_WANT_READ:
			// Something is blocking the operation . 
			// The docs say we should retry the operation.
			if (_isNonBlocking) {
				return totalReceived;
			}
			break;

		case SSL_ERROR_ZERO_RETURN:
			// Peer has closed the session
			return SOCKET_ERROR;
		case SSL_ERROR_WANT_CONNECT:
			return SOCKET_ERROR;
		case SSL_ERROR_WANT_ACCEPT:
			return SOCKET_ERROR;
		case SSL_ERROR_WANT_X509_LOOKUP:
			return SOCKET_ERROR;
		case SSL_ERROR_SYSCALL: {

			string pe = strerror(saved_errno);

			int error = ERR_get_error();
			if (error) {
				string pf = ERR_func_error_string(error);
				string pr = ERR_reason_error_string(error);

			} else {
			}

			return SOCKET_ERROR;
		}
		case SSL_ERROR_SSL: {

			string pe = strerror(saved_errno);

			int error = ERR_get_error();
			if (error) {
				string pf = ERR_func_error_string(error);
				string pr = ERR_reason_error_string(error);

			} else {
			}

			return SOCKET_ERROR;
		}
		default:
			return SOCKET_ERROR;
		}
	}

	return totalReceived;
}



INT32 OpenSSLSocket::close()
{
	if (_con) {
		SSL_shutdown(_con);
		SSL_free(_con);
		_con = NULL;
	}

	if(_peerCert) {
		X509_free(_peerCert);
		_peerCert = NULL;
	}

	if (_ctx) {
		SSL_CTX_free(_ctx);
		_ctx = NULL;
	}

	return TCPSocket::close();
}

#endif //_WIN32

//////////////////////////////////////////////////////////////////////////////////////////
//// SOCKS Socket classes
//////////////////////////////////////////////////////////////////////////////////////////
SOCKSSocket::SOCKSSocket(const ProxySettings &proxy_settings, string ca_file, string ca_path,
	string priv_cert, string cert_pass, string clientCert, IP_VERSION ipVersion, const bool enhancedTls
	, const bool tls, const string &cipher, const UINT16& localPort, const string& localIP) :
	TCPSocket(localPort, localIP, ipVersion), _proxy_settings(proxy_settings), _internal_socket(NULL),
	_ssl(false), _tls(false), _enhancedTls(false), _cipher(""), _caFile(""), _caPath(""), _privCert(""), _certPass(""), _clientCert("")
{
	if (proxy_settings.type == PROXY_NO_PROXY) {
		return;
	}

	_ssl = true;
	_enhancedTls = enhancedTls;
	_tls = tls;
	_cipher = cipher.empty() ? "ALL:eNULL:-AES256-SHA" : cipher;
	_caFile = ca_file;
	_caPath = ca_path;
	_privCert = priv_cert;
	_certPass = cert_pass;

	// Set SSL-SSPI client certificate subject name
	_clientCert = clientCert;
	

}

SOCKSSocket::SOCKSSocket(const ProxySettings &proxy_settings, IP_VERSION ipVersion, const UINT16& localPort, 
						 const string& localIP):
TCPSocket(localPort, localIP, ipVersion), _proxy_settings(proxy_settings), _internal_socket(NULL),
_ssl(false), _tls(false), _enhancedTls(false), _cipher(""), _caFile(""), _caPath(""), _privCert(""), _certPass(""), _clientCert("")
{
	if (proxy_settings.type == PROXY_NO_PROXY) {
		return;
	}

	_ssl = false;
	_enhancedTls = false;
}

SOCKSSocket::~SOCKSSocket()
{
	if (_internal_socket.get() != NULL) {
		_socket = INVALID_SOCKET;
	}
}

SOCKET_LIB_RESULT SOCKSSocket::connect(const string& ipAddr, const UINT16 port)
{

	if (_proxy_settings.type == PROXY_NO_PROXY) {
		//"connect SOCKS is disabled";
		return SOCKET_LIB_RES_SOCKS_ERROR;
	}

	if (_proxy_settings.type != PROXY_SOCKS_V5) {
		//"SOCKSSocket::connect only version 5 is supported";
		return SOCKET_LIB_RES_SOCKS_ERROR;
	}

	if (IPSocket::connect(_proxy_settings.server, _proxy_settings.port) != SOCKET_LIB_RES_OK) {
		// Failed to connect to proxy server
		return SOCKET_LIB_RES_SOCKET_ERROR;
	}
	
	// create a buffer that is big enough to hold the required parameters
	// notice that only the ipAddr field can be 255
	const UINT32 socksBufferSize = 1024;
	UINT8 socksBuf[socksBufferSize];
	UINT8 *pCurrent = socksBuf;
	UINT8 numOfMethods = 1;
	// Version identifier/method selection message
	*pCurrent = SOCKS_V5_PROTOCOL_VERSION; ++pCurrent;

	++pCurrent; // Meanwhile skip number of authentication methods
	*pCurrent = SOCKS_V5_METHOD_NO_AUTH; ++pCurrent;
	string user(_proxy_settings.user); 
	if (user.size() > 0) {
		*pCurrent = SOCKS_V5_METHOD_USER_PASS; ++pCurrent;
		++numOfMethods;
	}
	//*pCurrent = SOCKS_V5_METHOD_GSSAPI; ++pCurrent; change the above 2 to 3
	socksBuf[1] = numOfMethods; // Number of bytes that will follow (authentication methods)

	int res = IPSocket::send(socksBuf, static_cast<UINT32>(pCurrent - socksBuf));
	if (res != pCurrent - socksBuf) {
		//connect unable to send SOCKS5 Version"
		return SOCKET_LIB_RES_SOCKS_ERROR;
	}

	// Read server reply
	res = IPSocket::receive(socksBuf, 2, SOCKS_V5_TIMEOUT);
	if (res != 2) {
		//"SOCKSSocket::connect unable to receive SOCKS5 response"
		return SOCKET_LIB_RES_SOCKS_ERROR;
	}

	// Parse the server reply
	if (socksBuf[0] != SOCKS_V5_PROTOCOL_VERSION) {
		//SOCKSSocket::connect received invalid version
		return SOCKET_LIB_RES_SOCKS_ERROR;
	}

	bool auth = false;
	switch (socksBuf[1]) {
		case SOCKS_V5_METHOD_NO_AUTH:
			// No authentication needed
			auth = true;
			break;
		
		case SOCKS_V5_METHOD_USER_PASS:
			auth = _userPassAuth(_proxy_settings.user, _proxy_settings.password);
			break;

		
		case SOCKS_V5_METHOD_GSSAPI:
			//"connect authntication method GSSAPI is not supported"
			// auth = socks5GSSAuth();
			break;

		case SOCKS_V5_METHOD_NOT_ACCEPTABLE:
			//"connect authntication method was not acceptable by the SOCKS server"
			break;

		default:
			//"SOCKSSocket::connect invalid method"
			return SOCKET_LIB_RES_SOCKS_ERROR;
	}

	if (!auth) {
		// Authentication failed
		return SOCKET_LIB_RES_SOCKS_AUTH_FAIL;
	}

	// Request to connect message
	pCurrent = socksBuf;
	*pCurrent = SOCKS_V5_PROTOCOL_VERSION; ++pCurrent;
	*pCurrent = SOCKS_V5_CMD_CONNECT; ++pCurrent;
	*pCurrent = 0x00; ++pCurrent;
	*pCurrent = SOCKS_V5_ADDRESS_DOMAINNAME; ++pCurrent;
	*pCurrent = static_cast<UINT8>(ipAddr.length()); ++pCurrent;
	if (memcpy_s(pCurrent, ARRAYSIZE(socksBuf), ipAddr.c_str(), ipAddr.length()))
	{
		return SOCKET_LIB_RES_SOCKS_ERROR;
	}
	pCurrent += ipAddr.length();
	*((unsigned short*)pCurrent) = htons(port); pCurrent += sizeof(unsigned short);

	res = IPSocket::send(socksBuf, static_cast<UINT32>(pCurrent - socksBuf));
	if (res != pCurrent - socksBuf) {
		// Failed to send connect request
		return SOCKET_LIB_RES_SOCKS_ERROR;
	}
	
	// Recive response
	res = IPSocket::receive(socksBuf, 4, SOCKS_V5_TIMEOUT);
	if (res != 4) {
	// "connect unable to receive SOCKS5 response"
		return SOCKET_LIB_RES_SOCKS_ERROR;
	}

	// Parse the server reply
	if (socksBuf[0] != SOCKS_V5_PROTOCOL_VERSION) {
	//"SOCKSSocket::connect received invalid version"
		return SOCKET_LIB_RES_SOCKS_ERROR;
	}

	if (socksBuf[1] != SOCKS_V5_REP_SUCCEEDED) {
		// Got error response from server
		// "SOCKSSocket::connect failed to connect"
		switch(socksBuf[1]) {
			case SOCKS_V5_REP_ADDRESS_TYPE_NOT_SUPPORTED:
				break;
			case SOCKS_V5_REP_COMMAND_NOT_SUPPORTED:
				break;
			case SOCKS_V5_REP_CONNECTION_NOT_ALLOWED:
				break;
			case SOCKS_V5_REP_CONNECTION_REFUESED:
				break;
			case SOCKS_V5_REP_HOST_UNREACHABLE:
				break;
			case SOCKS_V5_REP_NETWORK_UNREACHABLE:
				break;
			case SOCKS_V5_REP_SERVER_FAILURE:
				break;
			case SOCKS_V5_REP_TTL_EXPIRED:
				break;
		}
		return SOCKET_LIB_RES_SOCKS_ERROR;
	}
	pCurrent = socksBuf + 4;
	int addLength = 0;
	switch (socksBuf[3]) {									// Case by address type
		case SOCKS_V5_ADDRESS_IPV4:							// IPv4 address
			addLength = 4 + 2;								// Read ip + port
			break;
		case SOCKS_V5_ADDRESS_DOMAINNAME:					// Domain name.
			if (IPSocket::receive(pCurrent, 1, SOCKS_V5_TIMEOUT) !=1) {
			//	connect unable to receive SOCKS5 response
				return SOCKET_LIB_RES_SOCKS_ERROR;
			}
			addLength = *pCurrent + 2;						// Read domain name + port
			++pCurrent;
			break;
		case SOCKS_V5_ADDRESS_IPV6:							// IPv6 address
			addLength = 16 + 2;								// Read ipv6 + port
			break;
	}
	if (addLength != IPSocket::receive(pCurrent, addLength, SOCKS_V5_TIMEOUT)) {
		//connect unable to receive SOCKS5 response
		return SOCKET_LIB_RES_SOCKS_ERROR;
	}
	
	if (_ssl) {

		SPtr<SSLSocket> sslSocket;

		sslSocket = SSLSocket::createSSLSocket(_caFile, _caPath, _privCert, _certPass, _clientCert, _ipVersion, _cipher, _enhancedTls, _socket);

		 SOCKET_LIB_RESULT res = sslSocket->handshake(ipAddr, port);
		 if (res != SOCKET_LIB_RES_OK) {
			 return res;
		 }
		_internal_socket = sslSocket;
	}
	else {
		_internal_socket = SPtr<Socket>(new TCPSocket(_socket, _ipVersion));
	}

	return SOCKET_LIB_RES_OK;
}

bool SOCKSSocket::_userPassAuth(string proxyUser, string proxyPass)
{
	if (proxyUser.empty() || proxyPass.empty()) {
		return false;
	}

	// create a buffer that is big enough to hold the auth message
	const UINT32 SOCKS_BUFF_SIZE = 1024;
	UINT8 socksBuf[SOCKS_BUFF_SIZE]; 
	UINT8 *pCurrent = socksBuf;

	// Username/password message
	*pCurrent = SOCKS_V5_USER_PWD_AUTH;
	++pCurrent;
	*pCurrent = static_cast<UINT8>(proxyUser.length());
	++pCurrent;
	if (memcpy_s(pCurrent, ARRAYSIZE(socksBuf), proxyUser.c_str(), proxyUser.length()))
	{
		return false;
	}
	pCurrent += proxyUser.length();
	*pCurrent = static_cast<UINT8>(proxyPass.length()); ++pCurrent;
	if (memcpy_s(pCurrent, ARRAYSIZE(socksBuf), proxyPass.c_str(), proxyPass.length()))
	{
		return false;
	}
	pCurrent += proxyPass.length();

	int res = IPSocket::send(socksBuf, static_cast<UINT32>(pCurrent - socksBuf));
	if (res != pCurrent - socksBuf) {
		// Failed to send auth request
		//_userPassAuth failed to send auth request
		return false;
	}

	// Read server reply
	res = IPSocket::receive(socksBuf, 2, SOCKS_V5_TIMEOUT);
	if (res != 2) {
		//_userPassAuth failed to receive auth reply
		return false;
	}

	// Parse the server reply
	if (socksBuf[0] != SOCKS_V5_USER_PWD_AUTH) {
		//_userPassAuth received invalid version
		return false;
	}

	return (socksBuf[1] == 0);
}

INT32 SOCKSSocket::send(const UINT8 * data, UINT32 dataLen)
{
	return (_internal_socket.get() != NULL) ? _internal_socket->send(data, dataLen) :	SOCKET_ERROR;
}

INT32 SOCKSSocket::receive(UINT8 *data, UINT32 dataLen, INT32 timeout)
{
	return (_internal_socket.get() != NULL) ? 
		_internal_socket->receive(data, dataLen, timeout) :	SOCKET_ERROR;
}

INT32 SOCKSSocket::close()
{
	return (_internal_socket.get() != NULL) ? _internal_socket->close() :	SOCKET_ERROR;
}

INT32 SOCKSSocket::getType()
{ 
	return (_internal_socket.get() != NULL) ? _internal_socket->getType() :	SOCKET_ERROR;;
}

bool SOCKSSocket::setNonBlocking( bool enable)
{
	TCPSocket::setNonBlocking(enable);
	return _internal_socket->setNonBlocking(enable);
}
