/****************************************************************
Copyright  2003-2011 Intel Corporation. All rights reserved.
*****************************************************************/

#include "StringUtils.h"
#include <Sddl.h>
#include <Wincrypt.h>
#include <algorithm>
#include <cctype> 
#include <iomanip>
static const wchar_t alphabet[] = L"0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
static const wchar_t DIVIDE_PLACE = L'-';
static const wchar_t CHECKSUM_PLACEHOLDER = L'$';
static const int base = ( sizeof(alphabet ) / sizeof(wchar_t) ) - 1;

using namespace std;

template <typename CTYPE>
bool StrUtl::Encrypt(const basic_string<CTYPE>& input, std::vector <unsigned char> &output, const bool& binary)
{
	if (input.empty())
	{
		output.clear();
		return true;
	}
	DATA_BLOB DataIn;
	DATA_BLOB DataOut;
	BYTE *pbDataInput =(BYTE *)input.c_str();
	DWORD cbDataInput = input.size()*sizeof(CTYPE);
	if (!binary)
	{
		cbDataInput++;
	}
	DataIn.pbData = pbDataInput;    
	DataIn.cbData = cbDataInput;


	if (CryptProtectData(
     &DataIn,
     NULL, // A description string
     NULL,                               // Optional entropy not used.
     NULL,                               // Reserved.
     NULL,                               // Pass NULL for the 
     CRYPTPROTECT_VERIFY_PROTECTION,
     &DataOut))
	{
		output.clear();
		output.assign(DataOut.pbData, DataOut.pbData + (size_t)DataOut.cbData);
		LocalFree(DataOut.pbData);
		return true;
	}
	else
		return false;


}
template <typename CTYPE>
bool StrUtl::Decrypt(const std::vector<unsigned char>& input, basic_string<CTYPE>& output)
{
	if (input.empty())
	{
		output.clear();
		return true;
	}

	DATA_BLOB DataIn;
	DATA_BLOB DataVerify;
	DataIn.cbData = input.size();
	DataIn.pbData = (BYTE *) &input.front();

	if (CryptUnprotectData(
			&DataIn,
			NULL,
			NULL,                 // Optional entropy
			NULL,                 // Reserved
			NULL,                 // Here, the optional 
			CRYPTPROTECT_UI_FORBIDDEN,
			&DataVerify))
	{
		output.clear();
		output = basic_string<CTYPE>((CTYPE *)DataVerify.pbData, (size_t)(DataVerify.cbData)/sizeof(CTYPE));
		// basic string
		//output = T((T *)DataVerify.pbData, (size_t)DataVerify.cbData/sizeof(T));
		// vector
		//output = T((T *)DataVerify.pbData, (T *)(DataVerify.pbData+DataVerify.cbData);
		SecureZeroMemory(DataVerify.pbData, DataVerify.cbData);
		LocalFree(DataVerify.pbData);
	}
	else
	{
		return false;
	}
	return true;
}
// Instantiate the template above, we use only 2 CTYPEs to it's not too bad

template bool StrUtl::Encrypt(const basic_string<char>& input, std::vector <unsigned char> &output, const bool& binary);
template bool StrUtl::Encrypt(const basic_string<wchar_t>& input, std::vector <unsigned char> &output, const bool& binary);
template bool StrUtl::Decrypt(const std::vector<unsigned char>& input, basic_string<char>& output);
template bool StrUtl::Decrypt(const std::vector<unsigned char>& input, basic_string<wchar_t>& output);

/*
 * String version of encryption. Notice that the encrypted string is NOT null
 * terminated and can include null characters. It is mandatory to use the
 * size() operator to determine the real size if needed.
 */
bool StrUtl::Encrypt(const std::string& input, std::string& output, const bool& binary)
{
	if (input.empty())
	{
		output.clear();
		return true;
	}
	DATA_BLOB DataIn;
	DATA_BLOB DataOut;
	BYTE *pbDataInput =(BYTE *)input.c_str();
	DWORD cbDataInput = input.size();
	if (!binary)
	{
		cbDataInput++;
	}
	DataIn.pbData = pbDataInput;    
	DataIn.cbData = cbDataInput;


	if (CryptProtectData(
     &DataIn,
     NULL, // A description string
     NULL,                               // Optional entropy not used.
     NULL,                               // Reserved.
     NULL,                               // Pass NULL for the 
     CRYPTPROTECT_VERIFY_PROTECTION,
     &DataOut))
	{
		output.clear();
		output.assign(DataOut.pbData, DataOut.pbData + (size_t)DataOut.cbData);
		LocalFree(DataOut.pbData);
		return true;
	}
	else
		return false;


}

/* See comment before Encrypt */
bool StrUtl::Decrypt(const std::string& input,std::string &output)
{
	if (input.empty())
	{
		output.clear();
		return true;
	}

	DATA_BLOB DataIn;
	DATA_BLOB DataVerify;
	DataIn.cbData = input.size();
	DataIn.pbData = (BYTE *)input.c_str();

	if (CryptUnprotectData(
			&DataIn,
			NULL,
			NULL,                 // Optional entropy
			NULL,                 // Reserved
			NULL,                 // Here, the optional 
			CRYPTPROTECT_UI_FORBIDDEN,
			&DataVerify))
	{
		output = (char *)DataVerify.pbData;
		SecureZeroMemory(DataVerify.pbData, DataVerify.cbData);
		LocalFree(DataVerify.pbData);
	}
	else
	{
		return false;
	}
	return true;
}


wstring StrUtl::cleanNonPrintable(wstring text)
{
	wstring printable_text;
	for(unsigned int i = 0; i < text.size(); i++)
	{
		try 
		{
			wchar_t c = text.at(i);
			//if( c < 0x0 || c > 0xFF ) // check the range of printable characters
			//	continue;
			if( iswprint( c ) )
				printable_text.push_back( c );
		}
		catch(...)
		{
			;//Ignore
		}
	}
	return printable_text;
}



bool StrUtl::getRandomByte(BYTE &RandomBuf)
{
	HCRYPTPROV hProv = NULL;
	RandomBuf = NULL;

	// Try to acquire context to Crypto provider.
	if (!CryptAcquireContext(&hProv, NULL, NULL, PROV_RSA_FULL, CRYPT_VERIFYCONTEXT))
	{
		if (GetLastError() == NTE_BAD_KEYSET) //Test for non-existent keyset
		{
			if (!CryptAcquireContext(&hProv, NULL, NULL, PROV_RSA_FULL, CRYPT_SILENT | CRYPT_NEWKEYSET))
			{
				CryptReleaseContext(hProv, 0);
				return false;
			}
		}
		else
		{
			CryptReleaseContext(hProv, 0);
			return false;
		}
	}

	//Generate a random sequence.
	if (!CryptGenRandom(hProv, 1, &RandomBuf))
	{
		CryptReleaseContext(hProv, 0);
		return false;
	}

	CryptReleaseContext(hProv, 0);
	return true;
}

bool StrUtl::isStrongPassword(const wstring & pwd, const bool underscoreValid) {
//Must be at least 8 and at most 32 characters long. 
//Characters allowed are 7-bit ASCII characters in the values of 32-126 inclusive. The characters   ,  ,  and  :  are not allowed. 
//Must have at least one digit character (e.g. 0, 1,9) 
//Must have at least one 7-bit ASCII non-alphanumeric character (e.g. !, $) 
//Must contain both lower case and upper case Latin characters 
//Must contain both lower-case Latin (a, b, z) and upper case Latin (A, B,Z), or non ASCII characters (UTF+00800 and above) 
	size_t len = pwd.length();
	if (len < 8) {
		return false; // Password length invalid
	}

	bool hasDigit = false;
	bool hasUpper = false;
	bool hasLower = false;
	bool hasNonAlpha = false;

	for (size_t i = 0; i < len; i++) {
		if (pwd[i] < L' ' || pwd[i] > L'~')
			return false; // Non printable character in password
		if (pwd[i] == L'\"' || pwd[i] == L',' || pwd[i] == L':')
			return false;
		if (pwd[i] >= L'0' && pwd[i] <= L'9')
			hasDigit = true;
		else if (pwd[i] >= L'a' && pwd[i] <= L'z')
			hasLower = true;
		else if (pwd[i] >= L'A' && pwd[i] <= L'Z')
			hasUpper = true;
		else if (underscoreValid || (!underscoreValid && pwd[i] != L'_'))
			hasNonAlpha = true;
	}

	return (hasDigit && hasLower && hasUpper && hasNonAlpha);
}


string StrUtl::ClearChars(const string& inString, const string& charsToClear)
{
	string result;

	string::size_type pos1 = inString.find_first_not_of(charsToClear);
	string::size_type pos2;
	string::size_type tokenLen;

	while(pos1 != string::npos)
	{
		pos2 = inString.find_first_of(charsToClear, pos1);
		tokenLen = ((string::npos == pos2) ? inString.length() : pos2) - pos1;
		result += inString.substr(pos1, tokenLen);
		pos1 = inString.find_first_not_of(charsToClear, pos2);
	}
	return result;
}

bool StrUtl::ContainsChar(const string& str, const string& chars)
{
	return string::npos != str.find_first_of(chars);
}

wstring StrUtl::wClearChars(const wstring& inString, const wstring& charsToClear)
{
	wstring result;

	wstring::size_type pos1 = inString.find_first_not_of(charsToClear);
	wstring::size_type pos2;
	wstring::size_type tokenLen;

	while(pos1 != wstring::npos)
	{
		pos2 = inString.find_first_of(charsToClear, pos1);
		tokenLen = ((wstring::npos == pos2) ? inString.length() : pos2) - pos1;
		result += inString.substr(pos1, tokenLen);
		pos1 = inString.find_first_not_of(charsToClear, pos2);
	}
	return result;
}



string StrUtl::ClearWhiteSpace(const string& inString)
{
	return ClearChars(inString, " \t\n\r");
}

wstring StrUtl::ClearWWhiteSpace(const wstring& inString)
{
	return wClearChars(inString, L" \t\n\r");
}

string StrUtl::TrimString(const string& inString)
{
	if(inString.empty())
	{
		return string();
	}
	string::size_type pos1 = inString.find_first_not_of(" \t\n\r");
	string::size_type pos2 = inString.find_last_not_of(" \t\n\r");
	return inString.substr(pos1 == string::npos ? 0 : pos1, 
		pos2 == string::npos ? inString.length() - 1 : pos2 - pos1 + 1);
}
wstring StrUtl::TrimWString(const wstring& inString)
{
	if(inString.empty())
	{
		return wstring();
	}
	wstring::size_type pos1 = inString.find_first_not_of(L" \t\n\r");
	wstring::size_type pos2 = inString.find_last_not_of(L" \t\n\r");
	return inString.substr(pos1 == wstring::npos ? 0 : pos1, 
		pos2 == wstring::npos ? inString.length() - 1 : pos2 - pos1 + 1);
}

bool StrUtl::CompareIgnoreCase(const string& string1, const string& string2)
{
	return _stricmp(string1.c_str(), string2.c_str()) == 0;
}

bool StrUtl::CompareIgnoreCase(const wstring& string1, const wstring& string2)
{
	return _wcsicmp(string1.c_str(), string2.c_str()) == 0;
}

bool StrUtl::IsDecimalUInt(const string& str)
{
	if(str.empty())
	{
		return false;
	}
	for(size_t i = 0; i < str.length(); i++)
	{
		if(str.at(i) < '0' || str.at(i) > '9')
		{
			return false;
		}
	}
	return true;
}

bool StrUtl::IsPrintableAscii(const string& str)
{
	if (str.empty())
	{
		return false;
	}
	for( size_t i = 0; i < str.length(); i++ )
	{
		if(str[i] < ' ' || str[i] > '~')
		{// non-printable ASCII
			return false;
		}
	}
	return true;
}

bool StrUtl::StartsWith(const string& str1, const string& str2)
{
	if (str1.find(str2) == 0 )
	{
		return true;
	}
	return false;
}



string StrUtl::ToUpper(const string& str)
{
	if (str.empty())
		return string();
	string tmp = str;
	std::transform(tmp.begin(), tmp.end(), tmp.begin(), (int(*)(int)) std::toupper);
	return tmp;
}

wstring StrUtl::ToWUpper(const wstring& str)
{
	if (str.empty())
		return wstring();
	wstring tmp = str;
	std::transform(tmp.begin(), tmp.end(), tmp.begin(), (int(*)(int)) towupper);
	return tmp;
}

string StrUtl::ToLower(const string& str)
{
	if (str.empty())
		return string();
	string tmp = str;
	std::transform(tmp.begin(), tmp.end(), tmp.begin(), (int(*)(int)) std::tolower);
	return tmp;
}

wstring StrUtl::ToWLower(const wstring& str)
{
	if (str.empty())
		return wstring();
	wstring tmp = str;
	std::transform(tmp.begin(), tmp.end(), tmp.begin(), (int(*)(int)) towlower);
	return tmp;
}


wstring StrUtl::GetDomainFromFqdn(const wstring& fqdn)
{
	if (fqdn.empty())
		return wstring();
	wstring domain = L"";

	string::size_type dot_pos = fqdn.find_first_of(L".");
	if(dot_pos != wstring::npos)
	{
		domain = fqdn.substr(dot_pos + 1);
	}

	return domain;
}

wstring StrUtl::GetHostNameFromFqdn(const wstring& fqdn)
{
	if (fqdn.empty())
		return wstring();
	wstring host = fqdn;

	string::size_type dot_pos = fqdn.find_first_of(L".");
	if(dot_pos != wstring::npos)
	{
		host = fqdn.substr(0,dot_pos);
	}

	return host;
}

/*
* Extracts a domain name from an LDAP object DN.
* Arguments:
*  DN	- distingushed name of object
* Return Value:
*  domain name.  call must free return 
*/

int StrUtl::CompareVersions(const string& version1,
									 const string& version2)
{
	string stringver1, stringver2;
	unsigned int ver1 = 0, ver2 = 0;
	size_t ind1 = 0;
	size_t ver1ind2 = 0, ver2ind2 = 0;
	while(ver1 == ver2 && ver1ind2 == ver2ind2)
	{
		ver1ind2 = version1.find(".", ind1);
		ver2ind2 = version2.find(".", ind1);
		if(ver1ind2 == string::npos)
		{
			StrUtl::FromString(version1.substr(ind1), ver1);
		}
		else
		{
			StrUtl::FromString(version1.substr(ind1, ver1ind2 - ind1), ver1);
		}

		if(ver2ind2 == string::npos)
		{
			StrUtl::FromString(version2.substr(ind1), ver2);
		}
		else
		{
			StrUtl::FromString(version2.substr(ind1, ver2ind2 - ind1), ver2);
		}
		if(ver1ind2 == string::npos)
		{
			break;
		}
		ind1 = ver1ind2 + 1;
	}

	return ver1 - ver2;
}

std::vector<unsigned char> StrUtl::ConvertStrToByteArray(const string & str)
{
	if (str.empty())
		return std::vector<unsigned char>();
	string tmp = StrUtl::DiscardNonHex(str);
	vector<unsigned char> res;
	int i, j;	

	string tmp1 = "";
	int len = tmp.size()/2;
	int hexVal;
	for(i = 0, j = 0; i < len; ++i)
	{
		tmp1 += tmp[j];
		tmp1 += tmp[j + 1];
		j += 2;
		//Convert to hex
		stringstream ss;
		ss.setf(std::ios_base::hex,std::ios_base::basefield);
		ss << tmp1;
		ss >> hexVal;
		tmp1.clear();
		res.push_back(hexVal);
	}
	return res;
}

string StrUtl::ConvertByteArrayToStr(const std::vector<unsigned char>& arr)
{
	if (arr.empty())
		return string();

	ostringstream oss;
	oss.setf(std::ios_base::hex,std::ios_base::basefield);
	for(size_t i = 0; i < arr.size()-1; i++)
	{
		oss << setw(2) << (int)(arr[i]);
		oss << ' ';
	}
	oss << setw(2) << (int)(arr.back());
	return oss.str();
}

string StrUtl::DiscardNonHex(const string & str)
{
	if (str.empty())
		return string();

	ostringstream oss;
	
	for(unsigned int i = 0; i < str.size(); i++)
	{
		if(isxdigit((unsigned char)str[i]))
		{
			if(((0==i) || (!isxdigit((unsigned char)str[i-1]))) &&
				((str.size()-1==i) || (!isxdigit((unsigned char)str[i+1]))))
			{
				oss <<hex << uppercase << setfill ('0')<< setw(2)<< str[i];
			}
			else
			{
				oss <<hex << uppercase << str[i];
			}
		}
	}

	string tmp = oss.str();
	// if odd number of characters, discard last character
	if (tmp.size() % 2 != 0)
	{
		tmp = tmp.substr(0,tmp.size() - 1);
	}
	return tmp;
}


wstring StrUtl::ExtractDomainFromADOU(const wstring& adou, const bool fqdnFormat)
{
	if (adou.empty())
		return wstring();

	wstring s;

	const wchar_t *p = adou.c_str();
	const wchar_t *q = NULL;
	while(*p)
	{
		q = wcschr(p, L',');
		if (_wcsnicmp(p, L"DC=", 3)==0) 
		{
			if(fqdnFormat)
			{
				p+=3;
			}
			if (q) {
				s.append(p, q);
				if(fqdnFormat)
				{
					s.append(L".");
				}
				else
				{
					s.append(L",");
				}
			}
			else
			{
				s.append(p);
				break;
			}
		}
		if (q)
		{
			p = q+1;
		}
		else 
		{
			break;
		}
	}

	
	return wstring(s.c_str());
}
