/****************************************************************
Copyright  2003-2011, Intel Corporation. All rights reserved.
*****************************************************************/
#include <sstream>
#include <iomanip>
#include "TypeDefinitions.h"
extern "C"
{
  #include "cencode.h"
  #include "cdecode.h"
}

using namespace TypeDefsNamespace;

// utility functions
static string ToBase64(unsigned char *in, int inlen)
{
 if(!in)
    {
      return "";
    }
  char* out = new char[((inlen + 1) * 4) / 3 + 3];
  char* out_start = out;

  // david code start here
  base64_encodestate state;
  base64_init_encodestate(&state);
  out += base64_encode_block((char *)in, inlen, out, &state);
  out += base64_encode_blockend(out, &state);

  *out = '\0';
  string tmp(out_start);
  delete[] out_start;
  return tmp;
}

static void FromBase64(const unsigned char * inStr, unsigned char * outStr, 
					   unsigned int  * outStrSize)
{
  if(!inStr || !outStr) return;
  
  base64_decodestate state;
  base64_init_decodestate(&state);
  *outStrSize = base64_decode_block((const char *)inStr, 
	  strlen((const char *)inStr), (char*)outStr, &state);
}

Base64::Base64()
{
  data = NULL;
  len = 0;
}

Base64::Base64(const Base64& b)
{
  data = NULL;
  len = 0;
  SetData(b.data, b.len);
}

Base64::Base64(const unsigned char *buffer, const unsigned int blen)
{
  data = NULL;
  len = 0;
  SetData(buffer, blen);
}

Base64::Base64(const string& input)
{
  data = NULL;
  len = 0;
  SetData(input);
}

Base64::~Base64()
{
  if(data)
    {
      delete[] data; 
      data = NULL;
    }
}

void Base64::ToVector(vector<unsigned char>& outVector ) const
{
	for (size_t i = 0 ; i < len ; i++)
	{
		outVector.push_back(data[i]);
	}
}

string Base64::ToString() const
{
  return ToBase64(data, len);
}
string Base64::ToHexString() const
{
	if ((NULL == data) || (0==len))
		return string();
	std::stringstream tmp;
	for (size_t i = 0; i<len;i++)
	{
		tmp << hex <<setfill('0') <<setw(2)<< ((unsigned int)data[i]);
	}
	return tmp.str();
}
void Base64::SetData(const string& input)
{
	unsigned char * temp, * tmp;
	unsigned int blen=0;
	tmp = new unsigned char[((input.length()) * 3/4 + 1)];
	FromBase64((const unsigned char*)input.c_str(), tmp, &blen);
	temp = data;
	len = blen; 
	data = tmp; 
	if(temp)
	{
		delete[] temp;
		temp = NULL;
	}
}

void Base64::SetData(const unsigned char *buffer, const unsigned int blen)
{
	if((buffer) && (data!=buffer))
	{
		unsigned char * temp, * tmp = new unsigned char[blen];
		if (memcpy_s(tmp, blen, buffer, blen))
		{
			printf("Error: Can't Copy Memory");
			delete[] tmp;
			return;
		}
		temp = data;
		len = blen; 
		data = tmp; 
		if(temp)
		{
			delete[] temp;
			temp = NULL;
		}
	}
}

Base64& Base64::operator =(const Base64& other)
{
  if (this == &other) {
    return *this;
  }
  if (data)
    {
      delete[] data;
      data = NULL;
    }
  len = other.len;
    if (len > 0) {
      data = new unsigned char[len];
	  if (memcpy_s(data, len, other.data, len))
	  {		  
		  printf("Error: Can't Copy Memory");
	  }
    }
    return *this;
}

bool Base64::Compare(const Base64& other) const
{
	bool ret = true;
	if(len != other.len)
	{
		ret = false;
	}
	else
	{
		for(size_t i = 0; i < len; i++)
		{
			if(data[i] !=  other.data[i])
			{
				ret = false;
				break;
			}
		}
	}
	return ret;
}