//  Copyright (C) 2007 Intel Corporation
#include "Logger.h"
#include "KVMLib.h"
#include "SPtr.h"
#include "SocketLib.h"
#include "ProxyManager.h"
#include <vector>
#include "SecString.h"

extern "C" EXPORTED_FUNC KVM_STATUS KVM_Init(char* root_certificate, 
						char* personal_certificate,
						char* certificate_pwd,
						ConnectionInfoCallback getdata)
{
	KVM_STATUS status  = KVM_STATUS_ERROR;
	Logger::EnableLogger(true);

	Logger::GetInstance() << "KVM_Init parameters:" << endl;
	Logger::GetInstance() << "\troot_certificate: " << root_certificate << endl;
	Logger::GetInstance() << "\tpersonal_certificate: " << personal_certificate << endl;
	Logger::GetInstance() << "\tConnectionInfo callback set to: " << getdata << endl;

	status = ProxyManager::GetInstance().Init(getdata);
	if(KVM_STATUS_OK == status)
	{
		string temp_string(certificate_pwd);
		SecString<char> cert_pwd(temp_string);
		if(!cert_pwd.isValid())
		{
			Logger::GetInstance() <<"Encrypt Password error: "<< GetLastError() <<endl;
			return KVM_STATUS_ERROR;
		}
		status = ProxyManager::GetInstance().SetCertificates(root_certificate, personal_certificate, cert_pwd);
	}
	

	Logger::GetInstance() << "KVM_Init return status: " << status << endl;

	return status;
}

extern "C" EXPORTED_FUNC KVM_STATUS KVM_SetCertificates(char* root_certificate, 
												char* personal_certificate,
												char* certificate_pwd)
{
	Logger::GetInstance() << "KVM_SetCertificates parameters:" << endl;
	Logger::GetInstance() << "\troot_certificate: " << root_certificate << endl;
	Logger::GetInstance() << "\tpersonal_certificate: " << personal_certificate << endl;
	string temp_string(certificate_pwd);
	SecString<char> cert_pwd(temp_string);
	if(!cert_pwd.isValid())
		{
			Logger::GetInstance() <<"Encrypt Password error: "<< GetLastError() <<endl;
			return KVM_STATUS_ERROR;
		}
	KVM_STATUS status = ProxyManager::GetInstance().SetCertificates(root_certificate, 
													  personal_certificate, 
													  cert_pwd);
	
	Logger::GetInstance() << "KVM_SetCertificates return status: " << status << endl;
	return status;
}

#ifdef _WIN32
extern "C" EXPORTED_FUNC KVM_STATUS KVM_SetClientCertificate(char* common_name)
{
	Logger::GetInstance() << "KVM_SetClientCertificate parameters:" << endl;
	Logger::GetInstance() << "\tcommon_name: " << common_name << endl;
	KVM_STATUS status;
	if(common_name != NULL)
		status = ProxyManager::GetInstance().SetClientCertificate(common_name);
	else
		status = ProxyManager::GetInstance().SetClientCertificate("");
	
	Logger::GetInstance() << "KVM_SetClientCertificate return status: " << status << endl;
	return status;
}
#endif

extern "C" EXPORTED_FUNC KVM_STATUS KVM_Start(unsigned short* port)
{
	return KVM_StartEx(port, NULL);
}

extern "C" EXPORTED_FUNC KVM_STATUS KVM_StartEx(unsigned short* port, const char* amt_version)
{
	if (NULL == port)
	{
		Logger::GetInstance() << "KVM_Start return KVM_STATUS_ERROR_INVALID_ARGUMENT" << endl;
		return KVM_STATUS_ERROR_INVALID_ARGUMENT;
	}
	Logger::GetInstance() << "KVM_Start parameters:" << endl;
	Logger::GetInstance() << "\tport: " << *port << endl;

	KVM_STATUS status = ProxyManager::GetInstance().Start(port, amt_version);

	Logger::GetInstance() << "KVM_Start return status: " << status << endl;
	return status;
}

extern "C" EXPORTED_FUNC KVM_STATUS KVM_Stop()
{
	Logger::GetInstance() << "KVM_Stop no parameters" << endl;
	KVM_STATUS status = ProxyManager::GetInstance().Stop();
	Logger::GetInstance() << "KVM_Stop return status: " << status << endl;
	return status;
}

extern "C" EXPORTED_FUNC KVM_STATUS KVM_Deinit()
{
	Logger::GetInstance() << "KVM_Deinit no parameters" << endl;
	KVM_STATUS status = ProxyManager::GetInstance().Deinit();
	Logger::GetInstance() << "KVM_Deinit return status: " << status << endl;
	return status;
}

extern "C" void KVM_RegisterStatusCallback(StatusCallback statusCallback)
{
	Logger::GetInstance() << "KVM_RegisterStatusCallback register statusCallback...." << endl;
	ProxyManager::GetInstance().RegisterStatusEvent(statusCallback);
}
