﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Intel.Management.Wsman;

namespace Intel.Management.PSModule.Amt
{
    class RedirectionService : SettingsContainer
    {

        public RedirectionService(DriveItem parent)
            : base("Redirection", parent)
        {
        }

        public override void GetChildItems(ChildWriter writer)
        {
            IWsmanConnection conn = ((AmtRootService)GetRoot()).Connection;
            _refToSettings= conn.NewReference("AMT_RedirectionService");
            _settingsObj= _refToSettings.Get();


            string state = _settingsObj.GetProperty("EnabledState").ToString();


            // add listern state
            writer.Add(new SolEnabled(state, this));
            writer.Add(new IderEnabled(state, this));
            writer.Add(new Listener(bool.Parse(_settingsObj.GetProperty("ListenerEnabled").ToString()), this));

        }

        public void RequestStateChange(string state)
        {
            IManagedInstance inputObj = _refToSettings.CreateMethodInput("RequestStateChange");
            inputObj.SetProperty("RequestedState", state);

            IManagedInstance outObj = _refToSettings.InvokeMethod(inputObj);

            switch (outObj.GetProperty("ReturnValue").ToString())
            {

                case "0":
                    _settingsObj.SetProperty("EnabledState", state);
                    break;
                case "1":
                    throw new WsmanInvokeException("Not Supported");
                case "2":
                    throw new WsmanInvokeException("Unknown or Unspecified Error");
                case "4":
                    throw new WsmanInvokeException("Failed");
                case "5":
                    throw new WsmanInvokeException("Invalid Parameter");
                case "4096":
                    throw new WsmanInvokeException("Method Parameters Checked - Job Started");
                case "4097":
                    throw new WsmanInvokeException("Invalid State Transition");
                case "4099":
                    throw new WsmanInvokeException("Busy");
                default:
                    throw new WsmanInvokeException("AMT error " + outObj.GetProperty("ReturnValue").ToString());
            }


        }





        class SolEnabled : SettingsItem
        {
            public SolEnabled(string state, SettingsContainer parent)
                : base("SolEnabled", parent)
            {
                if (state.Equals("32770") || state.Equals("32771"))
                    _value = true;
                else
                    _value = false;

            }

            public override void SetItem(object values, DriveProvider provider)
            {
                RedirectionService service = _parent as RedirectionService;
                string state = service.GetSetting("EnabledState");
                string changeTo = null;


                if (bool.Parse(values.ToString()))
                {
                    if (state.Equals("32768")) // both SOL and IDER are disabled
                        changeTo = "32770"; //enable just SOL
                    else if (state.Equals("32769")) // just IDER is enabled
                        changeTo = "32771"; //Enable both IDER and SOL
                }
                else
                {
                    if (state.Equals("32771")) //both IDER and SOL are enabled
                        changeTo = "32769"; //enabled just IDER
                    else if (state.Equals("32770")) // Just SOL is enabled
                        changeTo = "32768"; //Disable both IDER and SOL
                }
                if (changeTo != null)
                {
                    service.RequestStateChange(changeTo);
                    if (changeTo.Equals("32770") || changeTo.Equals("32771"))
                        _value = true;
                    else
                        _value = false;
                }
            }

        }

        class IderEnabled : SettingsItem
        {
            public IderEnabled(string state,SettingsContainer parent)
                : base("IderEnabled", parent)
            {
                if (state.Equals("32769") || state.Equals("32771"))
                    _value = true;
                else
                    _value = false;
            }

            public override void SetItem(object values, DriveProvider provider)
            {
                RedirectionService service = _parent as RedirectionService;
                string state = service.GetSetting("EnabledState");
                string changeTo=null;

                if (bool.Parse(values.ToString()) )
                {
                    if (state.Equals("32768")) // both SOL and IDER are disabled
                        changeTo = "32769"; //enable just IDER
                    else if (state.Equals("32770")) // just SOL is eabled
                        changeTo = "32771"; //Enable both IDER and SOL
                }
                else
                {
                    if (state.Equals("32769")) //just IDER is enabled
                        changeTo = "32768"; //disable both IDER and SOL
                    else if (state.Equals("32771")) // Both IDER and SOL are anabled
                        changeTo = "32770"; //Enable just SOL
                }
                if (changeTo != null)
                {
                    service.RequestStateChange(changeTo);
                    if (changeTo.Equals("32769") || changeTo.Equals("32771"))
                        _value = true;
                    else
                        _value = false;
                }
            }
        } // IderEnabled


        class Listener : BooleanSetting
        {
            public Listener(bool value, SettingsContainer parent)
                : base("ListenerEnabled",parent)
            {
                _value = value;
            }
          
        } // end listener claseee

   
    } //Redirection Service
}//end namespace 
