﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Intel.Management.Wsman;

namespace Intel.Management.PSModule.Amt
{
    class RedirectionServices : DriveContainer
    {

        enum DisplayType
        {
            Primary=0,
            Secondary=1
        }


        class DefaultScreen : DriveItem
        {

            IManagedInstance _settingsObj;

            public DefaultScreen(IManagedInstance settingsObj, DriveItem parent)
                : base("DefaultScreen", parent)
            {
                _value = (DisplayType)int.Parse(settingsObj.GetProperty("DefaultScreen").ToString());
                _settingsObj = settingsObj;
            }

            public override void SetItem(object values, DriveProvider provider)
            {
                IManagedReference settingsRef = _settingsObj.Connection.NewReference("IPS_KVMRedirectionSettingData");
                settingsRef.AddSelector("InstanceID", "Intel(r) KVM Redirection Settings");

                DisplayType displayType = DisplayType.Primary;

                if (values.ToString().ToLower().Equals("primary") || values.ToString().Equals("0"))
                {
                    displayType = DisplayType.Primary;
                }
                else if (values.ToString().ToLower().StartsWith("Second") || values.ToString().Equals("1"))
                {
                    displayType = DisplayType.Secondary;
                }
                _settingsObj.SetProperty("DefaultScreen", displayType.ToString());
                _settingsObj = settingsRef.Put(_settingsObj);
            }
        }


        class OptInTimeout : DriveItem
        {

            IManagedInstance _settingsObj;

            public OptInTimeout(IManagedInstance settingsObj, DriveItem parent)
                : base("ConsentDisplayTimeout", parent)
            {
                int sec = int.Parse(settingsObj.GetProperty("OptInPolicyTimeout").ToString());
                _value = new TimeSpan(0, 0 , sec);
                _settingsObj = settingsObj;
            }

            public override void SetItem(object values, DriveProvider provider)
            {
                IManagedReference settingsRef = _settingsObj.Connection.NewReference("IPS_KVMRedirectionSettingData");
                settingsRef.AddSelector("InstanceID", "Intel(r) KVM Redirection Settings");

                TimeSpan span = TimeSpan.Parse(values.ToString());
                _settingsObj.SetProperty("OptInPolicyTimeout", span.TotalSeconds.ToString());

                _settingsObj = settingsRef.Put(_settingsObj);
            }
        }


        class SessionTimeout : DriveItem
        {

            IManagedInstance _settingsObj;

            public SessionTimeout(IManagedInstance settingsObj, DriveItem parent)
                : base("SessionTimeout", parent)
            {
                 int min = int.Parse( settingsObj.GetProperty("SessionTimeout").ToString() );
                 _value = new TimeSpan(0, min, 0);
                _settingsObj = settingsObj;
            }

            public override void SetItem(object values, DriveProvider provider)
            {
                IManagedReference settingsRef = _settingsObj.Connection.NewReference("IPS_KVMRedirectionSettingData");
                settingsRef.AddSelector("InstanceID", "Intel(r) KVM Redirection Settings");

                TimeSpan span = TimeSpan.Parse(values.ToString());
                _settingsObj.SetProperty("SessionTimeout", span.TotalMinutes.ToString());

                _settingsObj = settingsRef.Put(_settingsObj);
            }
        }


        class RFBPassword : DriveItem
        {

            IManagedInstance _settingsObj;

            public RFBPassword(IManagedInstance settingsObj, DriveItem parent)
                : base("RFBPassword", parent)
            {
                _value = string.Empty;
                _settingsObj = settingsObj;
            }

            public override void SetItem(object values, DriveProvider provider)
            {
                IManagedReference settingsRef = _settingsObj.Connection.NewReference("IPS_KVMRedirectionSettingData");
                settingsRef.AddSelector("InstanceID", "Intel(r) KVM Redirection Settings");
                _settingsObj.SetProperty("RFBPassword", values.ToString());
                _settingsObj = settingsRef.Put(_settingsObj);
            }
        }


        class StandardPort : DriveItem
        {

            IManagedInstance _settingsObj;

            public StandardPort(IManagedInstance settingsObj, DriveItem parent)
                : base("UseStandardPort", parent)
            {
                _value = bool.Parse(settingsObj.GetProperty("Is5900PortEnabled").ToString());
                _settingsObj = settingsObj;
            }

            public override void SetItem(object values, DriveProvider provider)
            {
                IManagedReference settingsRef = _settingsObj.Connection.NewReference("IPS_KVMRedirectionSettingData");
                settingsRef.AddSelector("InstanceID", "Intel(r) KVM Redirection Settings");
                _settingsObj.SetProperty("Is5900PortEnabled", bool.Parse(values.ToString()).ToString().ToLower());
                _settingsObj = settingsRef.Put(_settingsObj);
                _value = bool.Parse(values.ToString());
            }
        }
        
        class KvmOptIn : DriveItem
        {

            IManagedInstance _settingsObj;

            public KvmOptIn(IManagedInstance settingsObj, DriveItem parent)
                : base("ConsentRequired",  parent)
            {
                _value = bool.Parse(settingsObj.GetProperty("OptInPolicy").ToString());
                _settingsObj = settingsObj;
            }

            public override void SetItem(object values, DriveProvider provider)
            {
                IManagedReference settingsRef = _settingsObj.Connection.NewReference("IPS_KVMRedirectionSettingData");
                settingsRef.AddSelector("InstanceID", "Intel(r) KVM Redirection Settings");
                _settingsObj.SetProperty("OptInPolicy", bool.Parse(values.ToString()).ToString().ToLower() );
                _settingsObj = settingsRef.Put(_settingsObj);
                _value=bool.Parse(values.ToString());
            }
        }

        class KvmEnabled : DriveItem
        {
            public KvmEnabled(string state, DriveItem parent)
                : base("AccessPointEnabled", state.Equals("2") || state.Equals("6"), parent)
            {
            }

            public override void SetItem(object values, DriveProvider provider)
            {
                RedirectionServices service = _parent as RedirectionServices;
                if (bool.Parse(values.ToString()))
                    service.EnableSAP("2");
                else
                    service.EnableSAP("3");

                _value = bool.Parse(values.ToString());
            }
        }


        class KvmFolder : DriveContainer
        {
            string _enabledState;
            IWsmanConnection _conn;

            public KvmFolder(IManagedInstance kvmObj, DriveItem parent) : 
                base("KVM",parent)
            {
                
                _enabledState= kvmObj.GetProperty("EnabledState").ToString();
                _conn = kvmObj.Connection;
               
            }

            public override void GetChildItems(ChildWriter writer)
            {
                
                // if enabled state not cached then get it
                if (_enabledState == null)
                {
                    IWsmanEnumeration colObj = _conn.ExecQuery("SELECT * FROM CIM_KVMRedirectionSAP");
                    foreach (IWsmanItem item in colObj)
                    {
                        // KVM access point found so the system is KVM capabile
                        if (item.Object.GetProperty("Name").ToString().Equals("KVM Redirection Service Access Point"))
                        {
                            _enabledState = item.Object.GetProperty("EnabledState").ToString();
                        }
                    }
                }
                //
                writer.Add(new KvmEnabled(_enabledState, this));
                _enabledState = null;


                //get reference to KVM settings
                IManagedReference settingsRef = _conn.NewReference("IPS_KVMRedirectionSettingData");
                settingsRef.AddSelector("InstanceID","Intel(r) KVM Redirection Settings");
                IManagedInstance settingsObj= settingsRef.Get();

                writer.Add(new KvmOptIn(settingsObj,this));
                writer.Add(new OptInTimeout(settingsObj, this));
                writer.Add(new SessionTimeout(settingsObj, this));
                writer.Add(new DefaultScreen(settingsObj, this));
                writer.Add(new RFBPassword(settingsObj,this));
                writer.Add(new StandardPort(settingsObj, this));
            }

        }

        class SolEnabled : DriveItem
        {
            public SolEnabled(string state,DriveItem parent)
                : base("SolEnabled",parent)
            {
                if (state.Equals("32770") || state.Equals("32771"))
                    _value = true;
                else
                    _value = false;
                
            }

            public override void SetItem(object values, DriveProvider provider)
            {
                RedirectionServices service = _parent as RedirectionServices;
                string state = service.GetEnabledState();
                string changeTo = null;

                if (bool.Parse(values.ToString()))
                {
                    if (state.Equals("32768")) // both SOL and IDER are disabled
                        changeTo = "32770"; //enable just SOL
                    else if (state.Equals("32769")) // just IDER is enabled
                        changeTo = "32771"; //Enable both IDER and SOL
                }
                else
                {
                    if (state.Equals("32771")) //both IDER and SOL are enabled
                        changeTo = "32769"; //enabled just IDER
                    else if (state.Equals("32770")) // Just SOL is enabled
                        changeTo = "32768"; //Disable both IDER and SOL
                }
                if (changeTo != null)
                    service.SetEnabledState(state);
              
            }
        }

        class IderEnabled : DriveItem
        {
            public IderEnabled(string state,DriveItem parent)
                : base("IderEnabled", parent)
            {
                if (state.Equals("32769") || state.Equals("32771"))
                    _value = true;
                else
                    _value = false;
            }

            public override void SetItem(object values, DriveProvider provider)
            {
                RedirectionServices service = _parent as RedirectionServices;
                string state = service.GetEnabledState();
                string changeTo=null;

                if (bool.Parse(values.ToString()) )
                {
                    if (state.Equals("32768")) // both SOL and IDER are disabled
                        changeTo = "32769"; //enable just IDER
                    else if (state.Equals("32770")) // just SOL is eabled
                        changeTo = "32771"; //Enable both IDER and SOL
                }
                else
                {
                    if (state.Equals("32769")) //just IDER is enabled
                        changeTo = "32768"; //disable both IDER and SOL
                    else if (state.Equals("32771")) // Both IDER and SOL are anabled
                        changeTo = "32770"; //Enable just SOL
                }
                if (changeTo!=null)
                    service.SetEnabledState(state);
            }
        }


        class Listener : DriveItem
        {
            public Listener(bool value, DriveItem parent)
                : base("ListenerEnabled",value,parent)
            {
            }
                    
            public override void SetItem(object values, DriveProvider provider)
            {
                RedirectionServices service = _parent as RedirectionServices;
                service.SetRedirectionListener(bool.Parse(values.ToString()));
                _value = bool.Parse(values.ToString());
            }
        }

       



        IManagedInstance _redObj;
        IManagedReference _redService;



        //IManagedReference kvmService;
        //IManagedReference kvm

        public RedirectionServices(WsmanConnection conn,DriveItem parent) 
            : base("Redirection",parent)
        {
            _redService=conn.NewReference("AMT_RedirectionService");
           
        }

        public override void GetChildItems(ChildWriter writer)
        {
           
            _redObj = _redService.Get();
            
            
            // add listern state
            writer.Add(new SolEnabled(_redObj.GetProperty("EnabledState").ToString(), this));
            writer.Add(new IderEnabled(_redObj.GetProperty("EnabledState").ToString(), this));
            writer.Add(new Listener(bool.Parse(_redObj.GetProperty("ListenerEnabled").ToString()),this));

            IWsmanEnumeration colObj = _redService.Connection.ExecQuery("SELECT * FROM CIM_KVMRedirectionSAP");
            foreach (IWsmanItem item in colObj)
            {
                // KVM access point found so the system is KVM capabile
                if (item.Object.GetProperty("Name").ToString().Equals("KVM Redirection Service Access Point"))
                {
                    writer.Add(new KvmFolder(item.Object, this));
                }
            }


        }

        void EnableSAP(string state)
        {
            IManagedReference kvmRef= _redObj.Connection.NewReference("CIM_KVMRedirectionSAP");
            kvmRef.AddSelector("Name", "KVM Redirection Service Access Point");

            IManagedInstance inputObj = kvmRef.CreateMethodInput("RequestStateChange");
            inputObj.SetProperty("RequestedState", state);
            
            IManagedInstance outObj= kvmRef.InvokeMethod(inputObj);
            string result = outObj.GetProperty("ReturnValue").ToString();
            if (result.Equals("5"))
                AmtException.ThrowInvokeError(outObj, "Invalid Parameter");
            else if (result.Equals("32806"))
                AmtException.ThrowInvokeError(outObj, "Flash write limit exceeded");
            else if (result.Equals("34843"))
                AmtException.ThrowInvokeError(outObj, "Audit log failure");
        }


        string GetEnabledState()
        {
            return _redObj.GetProperty("EnabledState").ToString();
        }

        void SetEnabledState(string state)
        {
            _redObj.SetProperty("EnabledState", state);
            _redService.Put(_redObj);
        }

        void SetRedirectionListener(bool enabled)
        {
            _redObj.SetProperty("ListenerEnabled", enabled.ToString().ToLower());
            _redService.Put(_redObj);
        }
    }
}
