﻿using System;
using System.Collections;
using System.Runtime.InteropServices;
using System.Text;

namespace Intel.Management.Wsman
{

    /// <summary>
    /// Supports a simple iteration over a collection of items returned from a WsMan service 
    /// </summary>
    /// <remarks>See IWsmanItem</remarks>
    [InterfaceType(ComInterfaceType.InterfaceIsDual)]
    [Guid("5505BAA7-2BFA-45ab-9D08-CE8C687BB700")]
    [ComVisible(true)]
    public interface IWsmanEnumeration : IDisposable
    {
        /// <summary>
        /// Gets the Connection Object being used to retrieve the items from the service
        /// </summary>
        IWsmanConnection Connection { get; }

        /// <summary>
        /// Gets a value indicating whether there are more items in the collection
        /// </summary>
        bool HasNext { get;}

        /// <summary>
        /// Gets the next Item in the collection
        /// </summary>
        /// <returns>An IWsmanItem representing the next item in the collection</returns>
        IWsmanItem Next();

        /// <summary>
        /// Closes an incomplete enumeration
        /// </summary>
        /// <remarks></remarks>
        void Close();

        /// <summary>
        /// Returns an enumerator that iterates through a collection
        /// </summary>
        /// <returns>An IEnumerator object that can be used to iterate through the collection</returns>
        [DispId(-4)]
        IEnumerator GetEnumerator();

    }
}
